@@ -26,8 +26,6 @@ NS_IMPL_NS_NEW_HTML_ELEMENT(Shared)
 
 namespace mozilla::dom {
 
-extern nsAttrValue::EnumTable kListTypeTable[];
-
 HTMLSharedElement::~HTMLSharedElement() = default;
 
 NS_IMPL_ELEMENT_CLONE(HTMLSharedElement)
@@ -69,62 +67,6 @@ void HTMLSharedElement::DoneAddingChildren(bool aHaveNotified) {
   }
 }
 
-bool HTMLSharedElement::ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
-                                       const nsAString& aValue,
-                                       nsIPrincipal* aMaybeScriptedPrincipal,
-                                       nsAttrValue& aResult) {
-  if (aNamespaceID == kNameSpaceID_None && mNodeInfo->Equals(nsGkAtoms::dir)) {
-    if (aAttribute == nsGkAtoms::type) {
-      return aResult.ParseEnumValue(aValue, mozilla::dom::kListTypeTable,
-                                    false);
-    }
-    if (aAttribute == nsGkAtoms::start) {
-      return aResult.ParseIntWithBounds(aValue, 1);
-    }
-  }
-
-  return nsGenericHTMLElement::ParseAttribute(aNamespaceID, aAttribute, aValue,
-                                              aMaybeScriptedPrincipal, aResult);
-}
-
-static void DirectoryMapAttributesIntoRule(
-    const nsMappedAttributes* aAttributes, MappedDeclarations& aDecls) {
-  if (!aDecls.PropertyIsSet(eCSSProperty_list_style_type)) {
-    // type: enum
-    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::type);
-    if (value) {
-      if (value->Type() == nsAttrValue::eEnum) {
-        aDecls.SetKeywordValue(eCSSProperty_list_style_type,
-                               value->GetEnumValue());
-      } else {
-        aDecls.SetKeywordValue(eCSSProperty_list_style_type,
-                               NS_STYLE_LIST_STYLE_DISC);
-      }
-    }
-  }
-
-  nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aDecls);
-}
-
-NS_IMETHODIMP_(bool)
-HTMLSharedElement::IsAttributeMapped(const nsAtom* aAttribute) const {
-  if (mNodeInfo->Equals(nsGkAtoms::dir)) {
-    static const MappedAttributeEntry attributes[] = {
-        {nsGkAtoms::type},
-        // { nsGkAtoms::compact }, // XXX
-        {nullptr}};
-
-    static const MappedAttributeEntry* const map[] = {
-        attributes,
-        sCommonAttributeMap,
-    };
-
-    return FindAttributeDependence(aAttribute, map);
-  }
-
-  return nsGenericHTMLElement::IsAttributeMapped(aAttribute);
-}
-
 static void SetBaseURIUsingFirstBaseWithHref(Document* aDocument,
                                              nsIContent* aMustMatch) {
   MOZ_ASSERT(aDocument, "Need a document!");
@@ -194,11 +136,11 @@ static void SetBaseTargetUsingFirstBaseWithTarget(Document* aDocument,
   aDocument->SetBaseTarget(u""_ns);
 }
 
-nsresult HTMLSharedElement::AfterSetAttr(int32_t aNamespaceID, nsAtom* aName,
-                                         const nsAttrValue* aValue,
-                                         const nsAttrValue* aOldValue,
-                                         nsIPrincipal* aSubjectPrincipal,
-                                         bool aNotify) {
+void HTMLSharedElement::AfterSetAttr(int32_t aNamespaceID, nsAtom* aName,
+                                     const nsAttrValue* aValue,
+                                     const nsAttrValue* aOldValue,
+                                     nsIPrincipal* aSubjectPrincipal,
+                                     bool aNotify) {
   if (aNamespaceID == kNameSpaceID_None) {
     if (aName == nsGkAtoms::href) {
       // If the href attribute of a <base> tag is changing, we may need to
@@ -260,15 +202,6 @@ void HTMLSharedElement::UnbindFromTree(bool aNullParent) {
   }
 }
 
-nsMapRuleToAttributesFunc HTMLSharedElement::GetAttributeMappingFunction()
-    const {
-  if (mNodeInfo->Equals(nsGkAtoms::dir)) {
-    return &DirectoryMapAttributesIntoRule;
-  }
-
-  return nsGenericHTMLElement::GetAttributeMappingFunction();
-}
-
 JSObject* HTMLSharedElement::WrapNode(JSContext* aCx,
                                       JS::Handle<JSObject*> aGivenProto) {
   if (mNodeInfo->Equals(nsGkAtoms::param)) {
