@@ -13,8 +13,7 @@
 
 class nsAttrValue;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 class MediaList;
 
@@ -30,12 +29,13 @@ class HTMLSourceElement final : public nsGenericHTMLElement {
 
   NS_IMPL_FROMNODE_HTML_WITH_TAG(HTMLSourceElement, source)
 
-  virtual nsresult Clone(dom::NodeInfo*, nsINode** aResult) const override;
+  nsresult Clone(dom::NodeInfo*, nsINode** aResult) const override;
 
   // Override BindToTree() so that we can trigger a load when we add a
   // child source element.
-  virtual nsresult BindToTree(Document* aDocument, nsIContent* aParent,
-                              nsIContent* aBindingParent) override;
+  nsresult BindToTree(BindContext&, nsINode& aParent) override;
+
+  void UnbindFromTree(bool aNullParent) override;
 
   // If this element's media attr matches for its owner document.  Returns true
   // if no media attr was set.
@@ -89,20 +89,55 @@ class HTMLSourceElement final : public nsGenericHTMLElement {
     SetHTMLAttr(nsGkAtoms::media, aMedia, rv);
   }
 
+  uint32_t Width() const {
+    return GetDimensionAttrAsUnsignedInt(nsGkAtoms::width, 0);
+  }
+  void SetWidth(uint32_t aWidth, ErrorResult& aRv) {
+    SetUnsignedIntAttr(nsGkAtoms::width, aWidth, 0, aRv);
+  }
+
+  uint32_t Height() const {
+    return GetDimensionAttrAsUnsignedInt(nsGkAtoms::height, 0);
+  }
+  void SetHeight(uint32_t aHeight, ErrorResult& aRv) {
+    SetUnsignedIntAttr(nsGkAtoms::height, aHeight, 0, aRv);
+  }
+
+  const nsMappedAttributes* GetAttributesMappedForImage() const {
+    return mMappedAttributesForImage;
+  }
+
+  static bool IsAttributeMappedToImages(const nsAtom* aAttribute) {
+    return aAttribute == nsGkAtoms::width || aAttribute == nsGkAtoms::height;
+  }
+
  protected:
   virtual ~HTMLSourceElement();
 
-  virtual JSObject* WrapNode(JSContext* aCx,
-                             JS::Handle<JSObject*> aGivenProto) override;
+  JSObject* WrapNode(JSContext* aCx,
+                     JS::Handle<JSObject*> aGivenProto) override;
 
- protected:
-  virtual nsresult AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
-                                const nsAttrValue* aValue,
-                                const nsAttrValue* aOldValue,
-                                nsIPrincipal* aMaybeScriptedPrincipal,
-                                bool aNotify) override;
+  bool ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
+                      const nsAString& aValue,
+                      nsIPrincipal* aMaybeScriptedPrincipal,
+                      nsAttrValue& aResult) override;
+
+  void AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
+                    const nsAttrValue* aValue, const nsAttrValue* aOldValue,
+                    nsIPrincipal* aMaybeScriptedPrincipal,
+                    bool aNotify) override;
 
  private:
+  // Generates a new MediaList using the given input
+  void UpdateMediaList(const nsAttrValue* aValue);
+
+  void BuildMappedAttributesForImage();
+
+  bool IsInPicture() const {
+    return GetParentElement() &&
+           GetParentElement()->IsHTMLElement(nsGkAtoms::picture);
+  }
+
   RefPtr<MediaList> mMediaList;
   RefPtr<MediaSource> mSrcMediaSource;
 
@@ -112,11 +147,11 @@ class HTMLSourceElement final : public nsGenericHTMLElement {
   // The triggering principal for the srcset attribute.
   nsCOMPtr<nsIPrincipal> mSrcsetTriggeringPrincipal;
 
-  // Generates a new MediaList using the given input
-  void UpdateMediaList(const nsAttrValue* aValue);
+  // The mapped attributes to HTMLImageElement if we are associated with a
+  // <picture> with a valid <img>.
+  RefPtr<nsMappedAttributes> mMappedAttributesForImage;
 };
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
 
 #endif  // mozilla_dom_HTMLSourceElement_h
