@@ -9,14 +9,14 @@
 #include "mozilla/MappedDeclarations.h"
 #include "nsAttrValueInlines.h"
 #include "nsMappedAttributes.h"
+#include "nsStyleConsts.h"
 #include "mozilla/dom/HTMLTableCaptionElementBinding.h"
 
 NS_IMPL_NS_NEW_HTML_ELEMENT(TableCaption)
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
-HTMLTableCaptionElement::~HTMLTableCaptionElement() {}
+HTMLTableCaptionElement::~HTMLTableCaptionElement() = default;
 
 JSObject* HTMLTableCaptionElement::WrapNode(JSContext* aCx,
                                             JS::Handle<JSObject*> aGivenProto) {
@@ -26,10 +26,8 @@ JSObject* HTMLTableCaptionElement::WrapNode(JSContext* aCx,
 NS_IMPL_ELEMENT_CLONE(HTMLTableCaptionElement)
 
 static const nsAttrValue::EnumTable kCaptionAlignTable[] = {
-    {"left", NS_STYLE_CAPTION_SIDE_LEFT},
-    {"right", NS_STYLE_CAPTION_SIDE_RIGHT},
-    {"top", NS_STYLE_CAPTION_SIDE_TOP},
-    {"bottom", NS_STYLE_CAPTION_SIDE_BOTTOM},
+    {"top", StyleCaptionSide::Top},
+    {"bottom", StyleCaptionSide::Bottom},
     {nullptr, 0}};
 
 bool HTMLTableCaptionElement::ParseAttribute(
@@ -72,5 +70,4 @@ nsMapRuleToAttributesFunc HTMLTableCaptionElement::GetAttributeMappingFunction()
   return &MapAttributesIntoRule;
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
