@@ -9,6 +9,8 @@
 #include "nsAttrValueInlines.h"
 #include "nsHTMLStyleSheet.h"
 #include "nsMappedAttributes.h"
+#include "nsWrapperCacheInlines.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/HTMLCollectionBinding.h"
 #include "mozilla/dom/HTMLTableElementBinding.h"
 #include "nsContentUtils.h"
@@ -16,8 +18,7 @@
 
 NS_IMPL_NS_NEW_HTML_ELEMENT(Table)
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 /* ------------------------- TableRowsCollection --------------------------- */
 /**
@@ -47,8 +48,8 @@ class TableRowsCollection final : public nsIHTMLCollection,
 
   NS_IMETHOD ParentDestroyed();
 
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_AMBIGUOUS(TableRowsCollection,
-                                                         nsIHTMLCollection)
+  NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS_AMBIGUOUS(TableRowsCollection,
+                                                        nsIHTMLCollection)
 
   // nsWrapperCache
   using nsWrapperCache::GetWrapperPreserveColor;
@@ -488,7 +489,7 @@ void TableRowsCollection::ContentRemoved(nsIContent* aChild,
   }
 }
 
-void TableRowsCollection::NodeWillBeDestroyed(const nsINode* aNode) {
+void TableRowsCollection::NodeWillBeDestroyed(nsINode* aNode) {
   // Set mInitialized to false so CleanUp doesn't try to remove our mutation
   // observer, as we're going away. CleanUp() will reset mInitialized to true as
   // it returns.
@@ -608,7 +609,7 @@ already_AddRefed<nsGenericHTMLElement> HTMLTableElement::CreateTFoot() {
     if (!foot) {
       return nullptr;
     }
-    AppendChildTo(foot, true);
+    AppendChildTo(foot, true, IgnoreErrors());
   }
 
   return foot.forget();
@@ -750,7 +751,7 @@ already_AddRefed<nsGenericHTMLElement> HTMLTableElement::InsertRow(
 
       rowGroup = NS_NewHTMLTableSectionElement(nodeInfo.forget());
       if (rowGroup) {
-        aError = AppendChildTo(rowGroup, true);
+        AppendChildTo(rowGroup, true, aError);
         if (aError.Failed()) {
           return nullptr;
         }
@@ -816,18 +817,13 @@ bool HTMLTableElement::ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
       return aResult.ParseNonNegativeIntValue(aValue);
     }
     if (aAttribute == nsGkAtoms::height) {
-      return aResult.ParseSpecialIntValue(aValue);
+      // Purposeful spec violation (spec says to use ParseNonzeroHTMLDimension)
+      // to stay compatible with our old behavior and other browsers.  See
+      // https://github.com/whatwg/html/issues/4715
+      return aResult.ParseHTMLDimension(aValue);
     }
     if (aAttribute == nsGkAtoms::width) {
-      if (aResult.ParseSpecialIntValue(aValue)) {
-        // treat 0 width as auto
-        nsAttrValue::ValueType type = aResult.Type();
-        return !(
-            (type == nsAttrValue::eInteger && aResult.GetIntegerValue() == 0) ||
-            (type == nsAttrValue::ePercent &&
-             aResult.GetPercentValue() == 0.0f));
-      }
-      return false;
+      return aResult.ParseNonzeroHTMLDimension(aValue);
     }
 
     if (aAttribute == nsGkAtoms::align) {
@@ -837,9 +833,6 @@ bool HTMLTableElement::ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
         aAttribute == nsGkAtoms::bordercolor) {
       return aResult.ParseColor(aValue);
     }
-    if (aAttribute == nsGkAtoms::hspace || aAttribute == nsGkAtoms::vspace) {
-      return aResult.ParseIntWithBounds(aValue, 0);
-    }
   }
 
   return nsGenericHTMLElement::ParseBackgroundAttribute(
@@ -860,8 +853,6 @@ void HTMLTableElement::MapAttributesIntoRule(
   // which *element* it's matching (style rules should not stop matching
   // when the display type is changed).
 
-  nsCompatibility mode = aDecls.Document()->GetCompatibilityMode();
-
   // cellspacing
   const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::cellspacing);
   if (value && value->Type() == nsAttrValue::eInteger &&
@@ -873,35 +864,13 @@ void HTMLTableElement::MapAttributesIntoRule(
   // illegal)
   value = aAttributes->GetAttr(nsGkAtoms::align);
   if (value && value->Type() == nsAttrValue::eEnum) {
-    if (value->GetEnumValue() == NS_STYLE_TEXT_ALIGN_CENTER ||
-        value->GetEnumValue() == NS_STYLE_TEXT_ALIGN_MOZ_CENTER) {
+    if (value->GetEnumValue() == uint8_t(StyleTextAlign::Center) ||
+        value->GetEnumValue() == uint8_t(StyleTextAlign::MozCenter)) {
       aDecls.SetAutoValueIfUnset(eCSSProperty_margin_left);
       aDecls.SetAutoValueIfUnset(eCSSProperty_margin_right);
     }
   }
 
-  // hspace is mapped into left and right margin,
-  // vspace is mapped into top and bottom margins
-  // - *** Quirks Mode only ***
-  if (eCompatibility_NavQuirks == mode) {
-    value = aAttributes->GetAttr(nsGkAtoms::hspace);
-
-    if (value && value->Type() == nsAttrValue::eInteger) {
-      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_left,
-                                  (float)value->GetIntegerValue());
-      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_right,
-                                  (float)value->GetIntegerValue());
-    }
-
-    value = aAttributes->GetAttr(nsGkAtoms::vspace);
-
-    if (value && value->Type() == nsAttrValue::eInteger) {
-      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_top,
-                                  (float)value->GetIntegerValue());
-      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_bottom,
-                                  (float)value->GetIntegerValue());
-    }
-  }
   // bordercolor
   value = aAttributes->GetAttr(nsGkAtoms::bordercolor);
   nscolor color;
@@ -942,8 +911,7 @@ HTMLTableElement::IsAttributeMapped(const nsAtom* aAttribute) const {
   static const MappedAttributeEntry attributes[] = {
       {nsGkAtoms::cellpadding}, {nsGkAtoms::cellspacing},
       {nsGkAtoms::border},      {nsGkAtoms::width},
-      {nsGkAtoms::height},      {nsGkAtoms::hspace},
-      {nsGkAtoms::vspace},
+      {nsGkAtoms::height},
 
       {nsGkAtoms::bordercolor},
 
@@ -1023,24 +991,21 @@ void HTMLTableElement::ReleaseInheritedAttributes() {
   NS_IF_RELEASE(mTableInheritedAttributes);
 }
 
-nsresult HTMLTableElement::BindToTree(Document* aDocument, nsIContent* aParent,
-                                      nsIContent* aBindingParent) {
+nsresult HTMLTableElement::BindToTree(BindContext& aContext, nsINode& aParent) {
   ReleaseInheritedAttributes();
-  nsresult rv =
-      nsGenericHTMLElement::BindToTree(aDocument, aParent, aBindingParent);
+  nsresult rv = nsGenericHTMLElement::BindToTree(aContext, aParent);
   NS_ENSURE_SUCCESS(rv, rv);
   BuildInheritedAttributes();
   return NS_OK;
 }
 
-void HTMLTableElement::UnbindFromTree(bool aDeep, bool aNullParent) {
+void HTMLTableElement::UnbindFromTree(bool aNullParent) {
   ReleaseInheritedAttributes();
-  nsGenericHTMLElement::UnbindFromTree(aDeep, aNullParent);
+  nsGenericHTMLElement::UnbindFromTree(aNullParent);
 }
 
-nsresult HTMLTableElement::BeforeSetAttr(int32_t aNameSpaceID, nsAtom* aName,
-                                         const nsAttrValueOrString* aValue,
-                                         bool aNotify) {
+void HTMLTableElement::BeforeSetAttr(int32_t aNameSpaceID, nsAtom* aName,
+                                     const nsAttrValue* aValue, bool aNotify) {
   if (aName == nsGkAtoms::cellpadding && aNameSpaceID == kNameSpaceID_None) {
     ReleaseInheritedAttributes();
   }
@@ -1048,11 +1013,11 @@ nsresult HTMLTableElement::BeforeSetAttr(int32_t aNameSpaceID, nsAtom* aName,
                                              aNotify);
 }
 
-nsresult HTMLTableElement::AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
-                                        const nsAttrValue* aValue,
-                                        const nsAttrValue* aOldValue,
-                                        nsIPrincipal* aSubjectPrincipal,
-                                        bool aNotify) {
+void HTMLTableElement::AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
+                                    const nsAttrValue* aValue,
+                                    const nsAttrValue* aOldValue,
+                                    nsIPrincipal* aSubjectPrincipal,
+                                    bool aNotify) {
   if (aName == nsGkAtoms::cellpadding && aNameSpaceID == kNameSpaceID_None) {
     BuildInheritedAttributes();
   }
@@ -1060,5 +1025,4 @@ nsresult HTMLTableElement::AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
       aNameSpaceID, aName, aValue, aOldValue, aSubjectPrincipal, aNotify);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
