@@ -39,6 +39,11 @@
 #include <algorithm>
 #include <limits>
 
+extern mozilla::LazyLogModule gMediaElementLog;
+#define LOG(msg, ...)                        \
+  MOZ_LOG(gMediaElementLog, LogLevel::Debug, \
+          ("HTMLVideoElement=%p, " msg, this, ##__VA_ARGS__))
+
 nsGenericHTMLElement* NS_NewHTMLVideoElement(
     already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,
     mozilla::dom::FromParser aFromParser) {
@@ -103,7 +108,8 @@ void HTMLVideoElement::UpdateMediaSize(const nsIntSize& aSize) {
   // If we have a clone target, we should update its size as well.
   if (mVisualCloneTarget) {
     Maybe<nsIntSize> newSize = Some(aSize);
-    mVisualCloneTarget->Invalidate(true, newSize, true);
+    mVisualCloneTarget->Invalidate(ImageSizeChanged::Yes, newSize,
+                                   ForceInvalidate::Yes);
   }
 }
 
@@ -135,9 +141,9 @@ Maybe<CSSIntSize> HTMLVideoElement::GetVideoSize() const {
   return Some(size);
 }
 
-void HTMLVideoElement::Invalidate(bool aImageSizeChanged,
-                                  Maybe<nsIntSize>& aNewIntrinsicSize,
-                                  bool aForceInvalidate) {
+void HTMLVideoElement::Invalidate(ImageSizeChanged aImageSizeChanged,
+                                  const Maybe<nsIntSize>& aNewIntrinsicSize,
+                                  ForceInvalidate aForceInvalidate) {
   HTMLMediaElement::Invalidate(aImageSizeChanged, aNewIntrinsicSize,
                                aForceInvalidate);
   if (mVisualCloneTarget) {
@@ -188,11 +194,9 @@ void HTMLVideoElement::UnbindFromTree(bool aNullParent) {
   if (mVisualCloneSource) {
     mVisualCloneSource->EndCloningVisually();
   } else if (mVisualCloneTarget) {
-    RefPtr<AsyncEventDispatcher> asyncDispatcher =
-        new AsyncEventDispatcher(this, u"MozStopPictureInPicture"_ns,
-                                 CanBubble::eNo, ChromeOnlyDispatch::eYes);
-    asyncDispatcher->RunDOMEventWhenSafe();
-
+    AsyncEventDispatcher::RunDOMEventWhenSafe(
+        *this, u"MozStopPictureInPicture"_ns, CanBubble::eNo,
+        ChromeOnlyDispatch::eYes);
     EndCloningVisually();
   }
 
@@ -215,13 +219,46 @@ bool HTMLVideoElement::IsInteractiveHTMLContent() const {
          HTMLMediaElement::IsInteractiveHTMLContent();
 }
 
+gfx::IntSize HTMLVideoElement::GetVideoIntrinsicDimensions() {
+  const auto& sz = mMediaInfo.mVideo.mDisplay;
+
+  // Prefer the size of the container as it's more up to date.
+  return ToMaybeRef(mVideoFrameContainer.get())
+      .map([&](auto& aVFC) { return aVFC.CurrentIntrinsicSize().valueOr(sz); })
+      .valueOr(sz);
+}
+
+uint32_t HTMLVideoElement::VideoWidth() {
+  if (!HasVideo()) {
+    return 0;
+  }
+  gfx::IntSize size = GetVideoIntrinsicDimensions();
+  if (mMediaInfo.mVideo.mRotation == VideoInfo::Rotation::kDegree_90 ||
+      mMediaInfo.mVideo.mRotation == VideoInfo::Rotation::kDegree_270) {
+    return size.height;
+  }
+  return size.width;
+}
+
+uint32_t HTMLVideoElement::VideoHeight() {
+  if (!HasVideo()) {
+    return 0;
+  }
+  gfx::IntSize size = GetVideoIntrinsicDimensions();
+  if (mMediaInfo.mVideo.mRotation == VideoInfo::Rotation::kDegree_90 ||
+      mMediaInfo.mVideo.mRotation == VideoInfo::Rotation::kDegree_270) {
+    return size.width;
+  }
+  return size.height;
+}
+
 uint32_t HTMLVideoElement::MozParsedFrames() const {
   MOZ_ASSERT(NS_IsMainThread(), "Should be on main thread.");
   if (!IsVideoStatsEnabled()) {
     return 0;
   }
 
-  if (nsContentUtils::ShouldResistFingerprinting(OwnerDoc())) {
+  if (OwnerDoc()->ShouldResistFingerprinting(RFPTarget::Unknown)) {
     return nsRFPService::GetSpoofedTotalFrames(TotalPlayTime());
   }
 
@@ -234,20 +271,20 @@ uint32_t HTMLVideoElement::MozDecodedFrames() const {
     return 0;
   }
 
-  if (nsContentUtils::ShouldResistFingerprinting(OwnerDoc())) {
+  if (OwnerDoc()->ShouldResistFingerprinting(RFPTarget::Unknown)) {
     return nsRFPService::GetSpoofedTotalFrames(TotalPlayTime());
   }
 
   return mDecoder ? mDecoder->GetFrameStatistics().GetDecodedFrames() : 0;
 }
 
-uint32_t HTMLVideoElement::MozPresentedFrames() const {
+uint32_t HTMLVideoElement::MozPresentedFrames() {
   MOZ_ASSERT(NS_IsMainThread(), "Should be on main thread.");
   if (!IsVideoStatsEnabled()) {
     return 0;
   }
 
-  if (nsContentUtils::ShouldResistFingerprinting(OwnerDoc())) {
+  if (OwnerDoc()->ShouldResistFingerprinting(RFPTarget::Unknown)) {
     return nsRFPService::GetSpoofedPresentedFrames(TotalPlayTime(),
                                                    VideoWidth(), VideoHeight());
   }
@@ -261,7 +298,7 @@ uint32_t HTMLVideoElement::MozPaintedFrames() {
     return 0;
   }
 
-  if (nsContentUtils::ShouldResistFingerprinting(OwnerDoc())) {
+  if (OwnerDoc()->ShouldResistFingerprinting(RFPTarget::Unknown)) {
     return nsRFPService::GetSpoofedPresentedFrames(TotalPlayTime(),
                                                    VideoWidth(), VideoHeight());
   }
@@ -274,7 +311,7 @@ double HTMLVideoElement::MozFrameDelay() {
   MOZ_ASSERT(NS_IsMainThread(), "Should be on main thread.");
 
   if (!IsVideoStatsEnabled() ||
-      nsContentUtils::ShouldResistFingerprinting(OwnerDoc())) {
+      OwnerDoc()->ShouldResistFingerprinting(RFPTarget::Unknown)) {
     return 0.0;
   }
 
@@ -311,7 +348,7 @@ HTMLVideoElement::GetVideoPlaybackQuality() {
     }
 
     if (mDecoder) {
-      if (nsContentUtils::ShouldResistFingerprinting(OwnerDoc())) {
+      if (OwnerDoc()->ShouldResistFingerprinting(RFPTarget::Unknown)) {
         totalFrames = nsRFPService::GetSpoofedTotalFrames(TotalPlayTime());
         droppedFrames = nsRFPService::GetSpoofedDroppedFrames(
             TotalPlayTime(), VideoWidth(), VideoHeight());
@@ -360,19 +397,28 @@ void HTMLVideoElement::UpdateWakeLock() {
 }
 
 bool HTMLVideoElement::ShouldCreateVideoWakeLock() const {
-  // Only request wake lock for video with audio or video from media stream,
-  // because non-stream video without audio is often used as a background image.
+  if (!StaticPrefs::media_video_wakelock()) {
+    return false;
+  }
+  // Only request wake lock for video with audio or video from media
+  // stream, because non-stream video without audio is often used as a
+  // background image.
   //
-  // Some web conferencing sites route audio outside the video element, and
-  // would not be detected unless we check for media stream, so do that below.
+  // Some web conferencing sites route audio outside the video element,
+  // and would not be detected unless we check for media stream, so do
+  // that below.
   //
-  // Media streams generally aren't used as background images, though if they
-  // were we'd get false positives. If this is an issue, we could check for
-  // media stream AND document has audio playing (but that was tricky to do).
+  // Media streams generally aren't used as background images, though if
+  // they were we'd get false positives. If this is an issue, we could
+  // check for media stream AND document has audio playing (but that was
+  // tricky to do).
   return HasVideo() && (mSrcStream || HasAudio());
 }
 
 void HTMLVideoElement::CreateVideoWakeLockIfNeeded() {
+  if (AppShutdown::IsInOrBeyond(ShutdownPhase::AppShutdownConfirmed)) {
+    return;
+  }
   if (!mScreenWakeLock && ShouldCreateVideoWakeLock()) {
     RefPtr<power::PowerManagerService> pmService =
         power::PowerManagerService::GetInstance();
@@ -600,4 +646,37 @@ void HTMLVideoElement::OnSecondaryVideoOutputFirstFrameRendered() {
       mVisualCloneTarget->GetVideoFrameContainer());
 }
 
+void HTMLVideoElement::OnVisibilityChange(Visibility aNewVisibility) {
+  HTMLMediaElement::OnVisibilityChange(aNewVisibility);
+
+  // See the alternative part after step 4, but we only pause/resume invisible
+  // autoplay for non-audible video, which is different from the spec. This
+  // behavior seems aiming to reduce the power consumption without interering
+  // users, and Chrome and Safari also chose to do that only for non-audible
+  // video, so we want to match them in order to reduce webcompat issue.
+  // https://html.spec.whatwg.org/multipage/media.html#ready-states:eligible-for-autoplay-2
+  if (!HasAttr(nsGkAtoms::autoplay) || IsAudible()) {
+    return;
+  }
+
+  if (aNewVisibility == Visibility::ApproximatelyVisible && mPaused &&
+      IsEligibleForAutoplay() && AllowedToPlay()) {
+    LOG("resume invisible paused autoplay video");
+    RunAutoplay();
+  }
+
+  // We need to consider the Pip window as well, which won't reflect in the
+  // visibility event.
+  if ((aNewVisibility == Visibility::ApproximatelyNonVisible &&
+       !IsCloningElementVisually()) &&
+      mCanAutoplayFlag) {
+    LOG("pause non-audible autoplay video when it's invisible");
+    PauseInternal();
+    mCanAutoplayFlag = true;
+    return;
+  }
+}
+
 }  // namespace mozilla::dom
+
+#undef LOG
