@@ -7,7 +7,9 @@
 #include "mozilla/dom/TextTrackManager.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/CycleCollectedJSContext.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Event.h"
 #include "mozilla/dom/HTMLMediaElement.h"
 #include "mozilla/dom/HTMLTrackElement.h"
@@ -30,8 +32,7 @@ mozilla::LazyLogModule gTextTrackLog("WebVTT");
   MOZ_LOG(gTextTrackLog, LogLevel::Verbose, \
           ("TextTrackManager=%p, " msg, this, ##__VA_ARGS__))
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 NS_IMPL_ISUPPORTS(TextTrackManager::ShutdownObserverProxy, nsIObserver);
 
@@ -44,11 +45,12 @@ CompareTextTracks::CompareTextTracks(HTMLMediaElement* aMediaElement) {
   mMediaElement = aMediaElement;
 }
 
-int32_t CompareTextTracks::TrackChildPosition(TextTrack* aTextTrack) const {
+Maybe<uint32_t> CompareTextTracks::TrackChildPosition(
+    TextTrack* aTextTrack) const {
   MOZ_DIAGNOSTIC_ASSERT(aTextTrack);
   HTMLTrackElement* trackElement = aTextTrack->GetTrackElement();
   if (!trackElement) {
-    return -1;
+    return Nothing();
   }
   return mMediaElement->ComputeIndexOf(trackElement);
 }
@@ -74,24 +76,26 @@ bool CompareTextTracks::LessThan(TextTrack* aOne, TextTrack* aTwo) const {
   TextTrackSource sourceTwo = aTwo->GetTextTrackSource();
   if (sourceOne != sourceTwo) {
     return sourceOne == TextTrackSource::Track ||
-           (sourceOne == AddTextTrack && sourceTwo == MediaResourceSpecific);
+           (sourceOne == TextTrackSource::AddTextTrack &&
+            sourceTwo == TextTrackSource::MediaResourceSpecific);
   }
   switch (sourceOne) {
-    case Track: {
-      int32_t positionOne = TrackChildPosition(aOne);
-      int32_t positionTwo = TrackChildPosition(aTwo);
-      // If either position one or positiontwo are -1 then something has gone
-      // wrong. In this case we should just put them at the back of the list.
-      return positionOne != -1 && positionTwo != -1 &&
-             positionOne < positionTwo;
+    case TextTrackSource::Track: {
+      Maybe<uint32_t> positionOne = TrackChildPosition(aOne);
+      Maybe<uint32_t> positionTwo = TrackChildPosition(aTwo);
+      // If either position one or positiontwo are Nothing then something has
+      // gone wrong. In this case we should just put them at the back of the
+      // list.
+      return positionOne.isSome() && positionTwo.isSome() &&
+             *positionOne < *positionTwo;
     }
-    case AddTextTrack:
+    case TextTrackSource::AddTextTrack:
       // For AddTextTrack sources the tracks will already be in the correct
       // relative order in the source array. Assume we're called in iteration
       // order and can therefore always report aOne < aTwo to maintain the
       // original temporal ordering.
       return true;
-    case MediaResourceSpecific:
+    case TextTrackSource::MediaResourceSpecific:
       // No rules for Media Resource Specific tracks yet.
       break;
   }
@@ -117,7 +121,6 @@ TextTrackManager::TextTrackManager(HTMLMediaElement* aMediaElement)
       mTimeMarchesOnDispatched(false),
       mUpdateCueDisplayDispatched(false),
       performedTrackSelection(false),
-      mCueTelemetryReported(false),
       mShutdown(false) {
   nsISupports* parentObject = mMediaElement->OwnerDoc()->GetParentObject();
 
@@ -160,13 +163,12 @@ already_AddRefed<TextTrack> TextTrackManager::AddTextTrack(
              NS_ConvertUTF16toUTF8(aLabel).get(),
              NS_ConvertUTF16toUTF8(aLanguage).get());
   AddCues(track);
-  ReportTelemetryForTrack(track);
 
   if (aTextTrackSource == TextTrackSource::Track) {
     RefPtr<nsIRunnable> task = NewRunnableMethod(
         "dom::TextTrackManager::HonorUserPreferencesForTrackSelection", this,
         &TextTrackManager::HonorUserPreferencesForTrackSelection);
-    nsContentUtils::RunInStableState(task.forget());
+    NS_DispatchToMainThread(task.forget());
   }
 
   return track.forget();
@@ -179,13 +181,12 @@ void TextTrackManager::AddTextTrack(TextTrack* aTextTrack) {
   WEBVTT_LOG("AddTextTrack TextTrack %p", aTextTrack);
   mTextTracks->AddTextTrack(aTextTrack, CompareTextTracks(mMediaElement));
   AddCues(aTextTrack);
-  ReportTelemetryForTrack(aTextTrack);
 
   if (aTextTrack->GetTextTrackSource() == TextTrackSource::Track) {
     RefPtr<nsIRunnable> task = NewRunnableMethod(
         "dom::TextTrackManager::HonorUserPreferencesForTrackSelection", this,
         &TextTrackManager::HonorUserPreferencesForTrackSelection);
-    nsContentUtils::RunInStableState(task.forget());
+    NS_DispatchToMainThread(task.forget());
   }
 }
 
@@ -240,40 +241,47 @@ void TextTrackManager::UpdateCueDisplay() {
   mUpdateCueDisplayDispatched = false;
 
   if (!mMediaElement || !mTextTracks || IsShutdown()) {
+    WEBVTT_LOG("Abort UpdateCueDisplay.");
     return;
   }
 
   nsIFrame* frame = mMediaElement->GetPrimaryFrame();
   nsVideoFrame* videoFrame = do_QueryFrame(frame);
   if (!videoFrame) {
+    WEBVTT_LOG("Abort UpdateCueDisplay, because of no video frame.");
     return;
   }
 
   nsCOMPtr<nsIContent> overlay = videoFrame->GetCaptionOverlay();
-  nsCOMPtr<nsIContent> controls = videoFrame->GetVideoControls();
   if (!overlay) {
+    WEBVTT_LOG("Abort UpdateCueDisplay, because of no overlay.");
     return;
   }
 
+  RefPtr<nsPIDOMWindowInner> window =
+      mMediaElement->OwnerDoc()->GetInnerWindow();
+  if (!window) {
+    WEBVTT_LOG("Abort UpdateCueDisplay, because of no window.");
+  }
+
   nsTArray<RefPtr<TextTrackCue>> showingCues;
   mTextTracks->GetShowingCues(showingCues);
 
-  if (showingCues.Length() > 0) {
-    WEBVTT_LOG("UpdateCueDisplay, processCues, showingCuesNum=%zu",
-               showingCues.Length());
-    RefPtr<nsVariantCC> jsCues = new nsVariantCC();
+  WEBVTT_LOG("UpdateCueDisplay, processCues, showingCuesNum=%zu",
+             showingCues.Length());
+  RefPtr<nsVariantCC> jsCues = new nsVariantCC();
+  jsCues->SetAsArray(nsIDataType::VTYPE_INTERFACE, &NS_GET_IID(EventTarget),
+                     showingCues.Length(),
+                     static_cast<void*>(showingCues.Elements()));
+  nsCOMPtr<nsIContent> controls = videoFrame->GetVideoControls();
 
-    jsCues->SetAsArray(nsIDataType::VTYPE_INTERFACE, &NS_GET_IID(EventTarget),
-                       showingCues.Length(),
-                       static_cast<void*>(showingCues.Elements()));
-    nsPIDOMWindowInner* window = mMediaElement->OwnerDoc()->GetInnerWindow();
-    if (window) {
-      sParserWrapper->ProcessCues(window, jsCues, overlay, controls);
-    }
-  } else if (overlay->Length() > 0) {
-    WEBVTT_LOG("UpdateCueDisplay EmptyString");
-    nsContentUtils::SetNodeTextContent(overlay, EmptyString(), true);
-  }
+  nsContentUtils::AddScriptRunner(NS_NewRunnableFunction(
+      "TextTrackManager::UpdateCueDisplay",
+      [window, jsCues, overlay, controls]() {
+        if (sParserWrapper) {
+          sParserWrapper->ProcessCues(window, jsCues, overlay, controls);
+        }
+      }));
 }
 
 void TextTrackManager::NotifyCueAdded(TextTrackCue& aCue) {
@@ -282,7 +290,6 @@ void TextTrackManager::NotifyCueAdded(TextTrackCue& aCue) {
     mNewCues->AddCue(aCue);
   }
   MaybeRunTimeMarchesOn();
-  ReportTelemetryForCue();
 }
 
 void TextTrackManager::NotifyCueRemoved(TextTrackCue& aCue) {
@@ -312,12 +319,11 @@ void TextTrackManager::PopulatePendingList() {
 
 void TextTrackManager::AddListeners() {
   if (mMediaElement) {
-    mMediaElement->AddEventListener(NS_LITERAL_STRING("resizevideocontrols"),
-                                    this, false, false);
-    mMediaElement->AddEventListener(NS_LITERAL_STRING("seeked"), this, false,
+    mMediaElement->AddEventListener(u"resizecaption"_ns, this, false, false);
+    mMediaElement->AddEventListener(u"resizevideocontrols"_ns, this, false,
                                     false);
-    mMediaElement->AddEventListener(NS_LITERAL_STRING("controlbarchange"), this,
-                                    false, true);
+    mMediaElement->AddEventListener(u"seeked"_ns, this, false, false);
+    mMediaElement->AddEventListener(u"controlbarchange"_ns, this, false, true);
   }
 }
 
@@ -417,14 +423,20 @@ TextTrackManager::HandleEvent(Event* aEvent) {
 
   nsAutoString type;
   aEvent->GetType(type);
-  if (type.EqualsLiteral("resizevideocontrols") ||
-      type.EqualsLiteral("seeked")) {
+  WEBVTT_LOG("Handle event %s", NS_ConvertUTF16toUTF8(type).get());
+
+  const bool setDirty = type.EqualsLiteral("seeked") ||
+                        type.EqualsLiteral("resizecaption") ||
+                        type.EqualsLiteral("resizevideocontrols");
+  const bool updateDisplay = type.EqualsLiteral("controlbarchange") ||
+                             type.EqualsLiteral("resizecaption");
+
+  if (setDirty) {
     for (uint32_t i = 0; i < mTextTracks->Length(); i++) {
       ((*mTextTracks)[i])->SetCuesDirty();
     }
   }
-
-  if (type.EqualsLiteral("controlbarchange")) {
+  if (updateDisplay) {
     UpdateCueDisplay();
   }
 
@@ -466,14 +478,14 @@ class SimpleTextTrackEvent : public Runnable {
 
 class CompareSimpleTextTrackEvents {
  private:
-  int32_t TrackChildPosition(SimpleTextTrackEvent* aEvent) const {
+  Maybe<uint32_t> TrackChildPosition(SimpleTextTrackEvent* aEvent) const {
     if (aEvent->mTrack) {
       HTMLTrackElement* trackElement = aEvent->mTrack->GetTrackElement();
       if (trackElement) {
         return mMediaElement->ComputeIndexOf(trackElement);
       }
     }
-    return -1;
+    return Nothing();
   }
   HTMLMediaElement* mMediaElement;
 
@@ -490,7 +502,8 @@ class CompareSimpleTextTrackEvents {
     // TimeMarchesOn step 13.1.
     if (aOne->mTime < aTwo->mTime) {
       return true;
-    } else if (aOne->mTime > aTwo->mTime) {
+    }
+    if (aOne->mTime > aTwo->mTime) {
       return false;
     }
 
@@ -508,7 +521,8 @@ class CompareSimpleTextTrackEvents {
       auto index2 = textTracks.IndexOf(t2);
       if (index1 < index2) {
         return true;
-      } else if (index1 > index2) {
+      }
+      if (index1 > index2) {
         return false;
       }
     }
@@ -520,12 +534,14 @@ class CompareSimpleTextTrackEvents {
     if (c1 != c2) {
       if (c1->StartTime() < c2->StartTime()) {
         return true;
-      } else if (c1->StartTime() > c2->StartTime()) {
+      }
+      if (c1->StartTime() > c2->StartTime()) {
         return false;
       }
       if (c1->EndTime() < c2->EndTime()) {
         return true;
-      } else if (c1->EndTime() > c2->EndTime()) {
+      }
+      if (c1->EndTime() > c2->EndTime()) {
         return false;
       }
 
@@ -536,7 +552,8 @@ class CompareSimpleTextTrackEvents {
       auto index2 = cues.IndexOf(c2);
       if (index1 < index2) {
         return true;
-      } else if (index1 > index2) {
+      }
+      if (index1 > index2) {
         return false;
       }
     }
@@ -751,7 +768,7 @@ void TextTrackManager::TimeMarchesOn() {
       WEBVTT_LOG("Prepare 'enter' event for cue %p [%f, %f] in missing cues",
                  cue, cue->StartTime(), cue->EndTime());
       SimpleTextTrackEvent* event = new SimpleTextTrackEvent(
-          NS_LITERAL_STRING("enter"), cue->StartTime(), cue->GetTrack(), cue);
+          u"enter"_ns, cue->StartTime(), cue->GetTrack(), cue);
       eventList.InsertElementSorted(
           event, CompareSimpleTextTrackEvents(mMediaElement));
       affectedTracks.AddTextTrack(cue->GetTrack(),
@@ -767,8 +784,8 @@ void TextTrackManager::TimeMarchesOn() {
           cue->StartTime() > cue->EndTime() ? cue->StartTime() : cue->EndTime();
       WEBVTT_LOG("Prepare 'exit' event for cue %p [%f, %f] in other cues", cue,
                  cue->StartTime(), cue->EndTime());
-      SimpleTextTrackEvent* event = new SimpleTextTrackEvent(
-          NS_LITERAL_STRING("exit"), time, cue->GetTrack(), cue);
+      SimpleTextTrackEvent* event =
+          new SimpleTextTrackEvent(u"exit"_ns, time, cue->GetTrack(), cue);
       eventList.InsertElementSorted(
           event, CompareSimpleTextTrackEvents(mMediaElement));
       affectedTracks.AddTextTrack(cue->GetTrack(),
@@ -784,7 +801,7 @@ void TextTrackManager::TimeMarchesOn() {
       WEBVTT_LOG("Prepare 'enter' event for cue %p [%f, %f] in current cues",
                  cue, cue->StartTime(), cue->EndTime());
       SimpleTextTrackEvent* event = new SimpleTextTrackEvent(
-          NS_LITERAL_STRING("enter"), cue->StartTime(), cue->GetTrack(), cue);
+          u"enter"_ns, cue->StartTime(), cue->GetTrack(), cue);
       eventList.InsertElementSorted(
           event, CompareSimpleTextTrackEvents(mMediaElement));
       affectedTracks.AddTextTrack(cue->GetTrack(),
@@ -802,10 +819,10 @@ void TextTrackManager::TimeMarchesOn() {
   for (uint32_t i = 0; i < affectedTracks.Length(); ++i) {
     TextTrack* ttrack = affectedTracks[i];
     if (ttrack) {
-      ttrack->DispatchAsyncTrustedEvent(NS_LITERAL_STRING("cuechange"));
+      ttrack->DispatchAsyncTrustedEvent(u"cuechange"_ns);
       HTMLTrackElement* trackElement = ttrack->GetTrackElement();
       if (trackElement) {
-        trackElement->DispatchTrackRunnable(NS_LITERAL_STRING("cuechange"));
+        trackElement->DispatchTrackRunnable(u"cuechange"_ns);
       }
     }
   }
@@ -837,25 +854,6 @@ void TextTrackManager::NotifyReset() {
   UpdateCueDisplay();
 }
 
-void TextTrackManager::ReportTelemetryForTrack(TextTrack* aTextTrack) const {
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aTextTrack);
-  MOZ_ASSERT(mTextTracks->Length() > 0);
-
-  TextTrackKind kind = aTextTrack->Kind();
-  Telemetry::Accumulate(Telemetry::WEBVTT_TRACK_KINDS, uint32_t(kind));
-}
-
-void TextTrackManager::ReportTelemetryForCue() {
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(!mNewCues->IsEmpty());
-
-  if (!mCueTelemetryReported) {
-    Telemetry::Accumulate(Telemetry::WEBVTT_USED_VTT_CUES, 1);
-    mCueTelemetryReported = true;
-  }
-}
-
 bool TextTrackManager::IsLoaded() {
   return mTextTracks ? mTextTracks->AreTextTracksLoaded() : true;
 }
@@ -877,5 +875,4 @@ void TextTrackManager::MaybeRunTimeMarchesOn() {
   TimeMarchesOn();
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
