@@ -4,97 +4,102 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "InputType.h"
+#include "mozilla/dom/InputType.h"
 
+#include "mozilla/Assertions.h"
+#include "mozilla/Likely.h"
 #include "nsIFormControl.h"
-#include "ButtonInputTypes.h"
-#include "CheckableInputTypes.h"
-#include "ColorInputType.h"
-#include "DateTimeInputTypes.h"
-#include "FileInputType.h"
-#include "HiddenInputType.h"
-#include "NumericInputTypes.h"
-#include "SingleLineTextInputTypes.h"
+#include "mozilla/dom/ButtonInputTypes.h"
+#include "mozilla/dom/CheckableInputTypes.h"
+#include "mozilla/dom/ColorInputType.h"
+#include "mozilla/dom/DateTimeInputTypes.h"
+#include "mozilla/dom/FileInputType.h"
+#include "mozilla/dom/HiddenInputType.h"
+#include "mozilla/dom/HTMLInputElement.h"
+#include "mozilla/dom/NumericInputTypes.h"
+#include "mozilla/dom/SingleLineTextInputTypes.h"
 
 #include "nsContentUtils.h"
 
-const mozilla::Decimal InputType::kStepAny = mozilla::Decimal(0);
+using namespace mozilla;
+using namespace mozilla::dom;
 
-/* static */ mozilla::UniquePtr<InputType, DoNotDelete> InputType::Create(
-    mozilla::dom::HTMLInputElement* aInputElement, uint8_t aType,
-    void* aMemory) {
-  mozilla::UniquePtr<InputType, DoNotDelete> inputType;
+const Decimal InputType::kStepAny = Decimal(0);
+
+/* static */ UniquePtr<InputType, InputType::DoNotDelete> InputType::Create(
+    HTMLInputElement* aInputElement, FormControlType aType, void* aMemory) {
+  UniquePtr<InputType, InputType::DoNotDelete> inputType;
   switch (aType) {
     // Single line text
-    case NS_FORM_INPUT_TEXT:
+    case FormControlType::InputText:
       inputType.reset(TextInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_TEL:
+    case FormControlType::InputTel:
       inputType.reset(TelInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_EMAIL:
+    case FormControlType::InputEmail:
       inputType.reset(EmailInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_SEARCH:
+    case FormControlType::InputSearch:
       inputType.reset(SearchInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_PASSWORD:
+    case FormControlType::InputPassword:
       inputType.reset(PasswordInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_URL:
+    case FormControlType::InputUrl:
       inputType.reset(URLInputType::Create(aInputElement, aMemory));
       break;
     // Button
-    case NS_FORM_INPUT_BUTTON:
+    case FormControlType::InputButton:
       inputType.reset(ButtonInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_SUBMIT:
+    case FormControlType::InputSubmit:
       inputType.reset(SubmitInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_IMAGE:
+    case FormControlType::InputImage:
       inputType.reset(ImageInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_RESET:
+    case FormControlType::InputReset:
       inputType.reset(ResetInputType::Create(aInputElement, aMemory));
       break;
     // Checkable
-    case NS_FORM_INPUT_CHECKBOX:
+    case FormControlType::InputCheckbox:
       inputType.reset(CheckboxInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_RADIO:
+    case FormControlType::InputRadio:
       inputType.reset(RadioInputType::Create(aInputElement, aMemory));
       break;
     // Numeric
-    case NS_FORM_INPUT_NUMBER:
+    case FormControlType::InputNumber:
       inputType.reset(NumberInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_RANGE:
+    case FormControlType::InputRange:
       inputType.reset(RangeInputType::Create(aInputElement, aMemory));
       break;
     // DateTime
-    case NS_FORM_INPUT_DATE:
+    case FormControlType::InputDate:
       inputType.reset(DateInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_TIME:
+    case FormControlType::InputTime:
       inputType.reset(TimeInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_MONTH:
+    case FormControlType::InputMonth:
       inputType.reset(MonthInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_WEEK:
+    case FormControlType::InputWeek:
       inputType.reset(WeekInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_DATETIME_LOCAL:
+    case FormControlType::InputDatetimeLocal:
       inputType.reset(DateTimeLocalInputType::Create(aInputElement, aMemory));
       break;
     // Others
-    case NS_FORM_INPUT_COLOR:
+    case FormControlType::InputColor:
       inputType.reset(ColorInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_FILE:
+    case FormControlType::InputFile:
       inputType.reset(FileInputType::Create(aInputElement, aMemory));
       break;
-    case NS_FORM_INPUT_HIDDEN:
+    case FormControlType::InputHidden:
       inputType.reset(HiddenInputType::Create(aInputElement, aMemory));
       break;
     default:
@@ -112,13 +117,10 @@ void InputType::GetNonFileValueInternal(nsAString& aValue) const {
   return mInputElement->GetNonFileValueInternal(aValue);
 }
 
-nsresult InputType::SetValueInternal(const nsAString& aValue, uint32_t aFlags) {
-  RefPtr<mozilla::dom::HTMLInputElement> inputElement(mInputElement);
-  return inputElement->SetValueInternal(aValue, aFlags);
-}
-
-mozilla::Decimal InputType::GetStepBase() const {
-  return mInputElement->GetStepBase();
+nsresult InputType::SetValueInternal(const nsAString& aValue,
+                                     const ValueSetterOptions& aOptions) {
+  RefPtr<HTMLInputElement> inputElement(mInputElement);
+  return inputElement->SetValueInternal(aValue, aOptions);
 }
 
 nsIFrame* InputType::GetPrimaryFrame() const {
@@ -138,138 +140,96 @@ bool InputType::IsValueMissing() const { return false; }
 
 bool InputType::HasTypeMismatch() const { return false; }
 
-bool InputType::HasPatternMismatch() const { return false; }
+Maybe<bool> InputType::HasPatternMismatch() const { return Some(false); }
 
 bool InputType::IsRangeOverflow() const { return false; }
 
 bool InputType::IsRangeUnderflow() const { return false; }
 
-bool InputType::HasStepMismatch(bool aUseZeroIfValueNaN) const { return false; }
+bool InputType::HasStepMismatch() const { return false; }
 
 bool InputType::HasBadInput() const { return false; }
 
 nsresult InputType::GetValidationMessage(
     nsAString& aValidationMessage,
     nsIConstraintValidation::ValidityStateType aType) {
-  nsresult rv = NS_OK;
+  aValidationMessage.Truncate();
 
   switch (aType) {
     case nsIConstraintValidation::VALIDITY_STATE_TOO_LONG: {
-      nsAutoString message;
       int32_t maxLength = mInputElement->MaxLength();
-      int32_t textLength =
-          mInputElement->InputTextLength(mozilla::dom::CallerType::System);
+      int32_t textLength = mInputElement->InputTextLength(CallerType::System);
       nsAutoString strMaxLength;
       nsAutoString strTextLength;
 
       strMaxLength.AppendInt(maxLength);
       strTextLength.AppendInt(textLength);
 
-      const char16_t* params[] = {strMaxLength.get(), strTextLength.get()};
-      rv = nsContentUtils::FormatLocalizedString(
-          nsContentUtils::eDOM_PROPERTIES, "FormValidationTextTooLong", params,
-          message);
-      aValidationMessage = message;
-      break;
+      return nsContentUtils::FormatMaybeLocalizedString(
+          aValidationMessage, nsContentUtils::eDOM_PROPERTIES,
+          "FormValidationTextTooLong", mInputElement->OwnerDoc(), strMaxLength,
+          strTextLength);
     }
     case nsIConstraintValidation::VALIDITY_STATE_TOO_SHORT: {
-      nsAutoString message;
       int32_t minLength = mInputElement->MinLength();
-      int32_t textLength =
-          mInputElement->InputTextLength(mozilla::dom::CallerType::System);
+      int32_t textLength = mInputElement->InputTextLength(CallerType::System);
       nsAutoString strMinLength;
       nsAutoString strTextLength;
 
       strMinLength.AppendInt(minLength);
       strTextLength.AppendInt(textLength);
 
-      const char16_t* params[] = {strMinLength.get(), strTextLength.get()};
-      rv = nsContentUtils::FormatLocalizedString(
-          nsContentUtils::eDOM_PROPERTIES, "FormValidationTextTooShort", params,
-          message);
-
-      aValidationMessage = message;
-      break;
-    }
-    case nsIConstraintValidation::VALIDITY_STATE_VALUE_MISSING: {
-      nsAutoString message;
-      rv = GetValueMissingMessage(message);
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-
-      aValidationMessage = message;
-      break;
+      return nsContentUtils::FormatMaybeLocalizedString(
+          aValidationMessage, nsContentUtils::eDOM_PROPERTIES,
+          "FormValidationTextTooShort", mInputElement->OwnerDoc(), strMinLength,
+          strTextLength);
     }
+    case nsIConstraintValidation::VALIDITY_STATE_VALUE_MISSING:
+      return GetValueMissingMessage(aValidationMessage);
     case nsIConstraintValidation::VALIDITY_STATE_TYPE_MISMATCH: {
-      nsAutoString message;
-      rv = GetTypeMismatchMessage(message);
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-
-      aValidationMessage = message;
-      break;
+      return GetTypeMismatchMessage(aValidationMessage);
     }
     case nsIConstraintValidation::VALIDITY_STATE_PATTERN_MISMATCH: {
-      nsAutoString message;
       nsAutoString title;
-      mInputElement->GetAttr(kNameSpaceID_None, nsGkAtoms::title, title);
+      mInputElement->GetAttr(nsGkAtoms::title, title);
+
       if (title.IsEmpty()) {
-        rv = nsContentUtils::GetLocalizedString(nsContentUtils::eDOM_PROPERTIES,
-                                                "FormValidationPatternMismatch",
-                                                message);
-      } else {
-        if (title.Length() >
-            nsIConstraintValidation::sContentSpecifiedMaxLengthMessage) {
-          title.Truncate(
-              nsIConstraintValidation::sContentSpecifiedMaxLengthMessage);
-        }
-        const char16_t* params[] = {title.get()};
-        rv = nsContentUtils::FormatLocalizedString(
-            nsContentUtils::eDOM_PROPERTIES,
-            "FormValidationPatternMismatchWithTitle", params, message);
-      }
-      aValidationMessage = message;
-      break;
-    }
-    case nsIConstraintValidation::VALIDITY_STATE_RANGE_OVERFLOW: {
-      nsAutoString message;
-      rv = GetRangeOverflowMessage(message);
-      if (NS_FAILED(rv)) {
-        return rv;
+        return nsContentUtils::GetMaybeLocalizedString(
+            nsContentUtils::eDOM_PROPERTIES, "FormValidationPatternMismatch",
+            mInputElement->OwnerDoc(), aValidationMessage);
       }
 
-      aValidationMessage = message;
-      break;
-    }
-    case nsIConstraintValidation::VALIDITY_STATE_RANGE_UNDERFLOW: {
-      nsAutoString message;
-      rv = GetRangeUnderflowMessage(message);
-      if (NS_FAILED(rv)) {
-        return rv;
+      if (title.Length() >
+          nsIConstraintValidation::sContentSpecifiedMaxLengthMessage) {
+        title.Truncate(
+            nsIConstraintValidation::sContentSpecifiedMaxLengthMessage);
       }
-
-      aValidationMessage = message;
-      break;
+      return nsContentUtils::FormatMaybeLocalizedString(
+          aValidationMessage, nsContentUtils::eDOM_PROPERTIES,
+          "FormValidationPatternMismatchWithTitle", mInputElement->OwnerDoc(),
+          title);
     }
+    case nsIConstraintValidation::VALIDITY_STATE_RANGE_OVERFLOW:
+      return GetRangeOverflowMessage(aValidationMessage);
+    case nsIConstraintValidation::VALIDITY_STATE_RANGE_UNDERFLOW:
+      return GetRangeUnderflowMessage(aValidationMessage);
     case nsIConstraintValidation::VALIDITY_STATE_STEP_MISMATCH: {
-      nsAutoString message;
+      Decimal value = mInputElement->GetValueAsDecimal();
+      if (MOZ_UNLIKELY(NS_WARN_IF(value.isNaN()))) {
+        // TODO(bug 1651070): This should ideally never happen, but we don't
+        // deal with lang changes correctly, so it could.
+        return GetBadInputMessage(aValidationMessage);
+      }
 
-      mozilla::Decimal value = mInputElement->GetValueAsDecimal();
-      MOZ_ASSERT(!value.isNaN());
+      Decimal step = mInputElement->GetStep();
+      MOZ_ASSERT(step != kStepAny && step > Decimal(0));
 
-      mozilla::Decimal step = mInputElement->GetStep();
-      MOZ_ASSERT(step != kStepAny && step > mozilla::Decimal(0));
+      Decimal stepBase = mInputElement->GetStepBase();
 
-      mozilla::Decimal stepBase = mInputElement->GetStepBase();
+      Decimal valueLow = value - NS_floorModulo(value - stepBase, step);
+      Decimal valueHigh = value + step - NS_floorModulo(value - stepBase, step);
 
-      mozilla::Decimal valueLow =
-          value - NS_floorModulo(value - stepBase, step);
-      mozilla::Decimal valueHigh =
-          value + step - NS_floorModulo(value - stepBase, step);
-
-      mozilla::Decimal maximum = mInputElement->GetMaximum();
+      Decimal maximum = mInputElement->GetMaximum();
 
       if (maximum.isNaN() || valueHigh <= maximum) {
         nsAutoString valueLowStr, valueHighStr;
@@ -277,49 +237,37 @@ nsresult InputType::GetValidationMessage(
         ConvertNumberToString(valueHigh, valueHighStr);
 
         if (valueLowStr.Equals(valueHighStr)) {
-          const char16_t* params[] = {valueLowStr.get()};
-          rv = nsContentUtils::FormatLocalizedString(
-              nsContentUtils::eDOM_PROPERTIES,
-              "FormValidationStepMismatchOneValue", params, message);
-        } else {
-          const char16_t* params[] = {valueLowStr.get(), valueHighStr.get()};
-          rv = nsContentUtils::FormatLocalizedString(
-              nsContentUtils::eDOM_PROPERTIES, "FormValidationStepMismatch",
-              params, message);
+          return nsContentUtils::FormatMaybeLocalizedString(
+              aValidationMessage, nsContentUtils::eDOM_PROPERTIES,
+              "FormValidationStepMismatchOneValue", mInputElement->OwnerDoc(),
+              valueLowStr);
         }
-      } else {
-        nsAutoString valueLowStr;
-        ConvertNumberToString(valueLow, valueLowStr);
-
-        const char16_t* params[] = {valueLowStr.get()};
-        rv = nsContentUtils::FormatLocalizedString(
-            nsContentUtils::eDOM_PROPERTIES,
-            "FormValidationStepMismatchOneValue", params, message);
+        return nsContentUtils::FormatMaybeLocalizedString(
+            aValidationMessage, nsContentUtils::eDOM_PROPERTIES,
+            "FormValidationStepMismatch", mInputElement->OwnerDoc(),
+            valueLowStr, valueHighStr);
       }
 
-      aValidationMessage = message;
-      break;
-    }
-    case nsIConstraintValidation::VALIDITY_STATE_BAD_INPUT: {
-      nsAutoString message;
-      rv = GetBadInputMessage(message);
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
+      nsAutoString valueLowStr;
+      ConvertNumberToString(valueLow, valueLowStr);
 
-      aValidationMessage = message;
-      break;
+      return nsContentUtils::FormatMaybeLocalizedString(
+          aValidationMessage, nsContentUtils::eDOM_PROPERTIES,
+          "FormValidationStepMismatchOneValue", mInputElement->OwnerDoc(),
+          valueLowStr);
     }
+    case nsIConstraintValidation::VALIDITY_STATE_BAD_INPUT:
+      return GetBadInputMessage(aValidationMessage);
     default:
+      MOZ_ASSERT_UNREACHABLE("Unknown validity state");
       return NS_ERROR_UNEXPECTED;
   }
-
-  return rv;
 }
 
 nsresult InputType::GetValueMissingMessage(nsAString& aMessage) {
-  return nsContentUtils::GetLocalizedString(
-      nsContentUtils::eDOM_PROPERTIES, "FormValidationValueMissing", aMessage);
+  return nsContentUtils::GetMaybeLocalizedString(
+      nsContentUtils::eDOM_PROPERTIES, "FormValidationValueMissing",
+      mInputElement->OwnerDoc(), aMessage);
 }
 
 nsresult InputType::GetTypeMismatchMessage(nsAString& aMessage) {
@@ -338,16 +286,14 @@ nsresult InputType::GetBadInputMessage(nsAString& aMessage) {
   return NS_ERROR_UNEXPECTED;
 }
 
-nsresult InputType::MinMaxStepAttrChanged() { return NS_OK; }
-
 bool InputType::ConvertStringToNumber(nsAString& aValue,
-                                      mozilla::Decimal& aResultValue) const {
+                                      Decimal& aResultValue) const {
   NS_WARNING("InputType::ConvertStringToNumber called");
 
   return false;
 }
 
-bool InputType::ConvertNumberToString(mozilla::Decimal aValue,
+bool InputType::ConvertNumberToString(Decimal aValue,
                                       nsAString& aResultString) const {
   NS_WARNING("InputType::ConvertNumberToString called");
 
@@ -365,7 +311,7 @@ bool InputType::ParseDate(const nsAString& aValue, uint32_t* aYear,
 
 bool InputType::ParseTime(const nsAString& aValue, uint32_t* aResult) const {
   // see comment in InputType::ParseDate().
-  return mozilla::dom::HTMLInputElement::ParseTime(aValue, aResult);
+  return HTMLInputElement::ParseTime(aValue, aResult);
 }
 
 bool InputType::ParseMonth(const nsAString& aValue, uint32_t* aYear,
