@@ -10,8 +10,8 @@
 #include "mozilla/dom/PerformanceBinding.h"
 #include "mozilla/dom/PerformanceEntryBinding.h"
 #include "mozilla/dom/PerformanceObserverBinding.h"
-#include "mozilla/dom/WorkerPrivate.h"
 #include "mozilla/dom/WorkerScope.h"
+#include "mozilla/StaticPrefs_dom.h"
 #include "nsIScriptError.h"
 #include "nsPIDOMWindow.h"
 #include "nsQueryObject.h"
@@ -22,7 +22,7 @@
 using namespace mozilla;
 using namespace mozilla::dom;
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(PerformanceObserver)
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(PerformanceObserver)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(PerformanceObserver)
   tmp->Disconnect();
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mCallback)
@@ -37,7 +37,6 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(PerformanceObserver)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mOwner)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mQueuedEntries)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-NS_IMPL_CYCLE_COLLECTION_TRACE_WRAPPERCACHE(PerformanceObserver)
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(PerformanceObserver)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(PerformanceObserver)
@@ -121,22 +120,20 @@ void PerformanceObserver::Notify() {
 
 void PerformanceObserver::QueueEntry(PerformanceEntry* aEntry) {
   MOZ_ASSERT(aEntry);
+  MOZ_ASSERT(ObservesTypeOfEntry(aEntry));
 
-  if (!ObservesTypeOfEntry(aEntry)) {
-    return;
-  }
   mQueuedEntries.AppendElement(aEntry);
 }
 
+static constexpr nsLiteralString kValidEventTimingNames[2] = {
+    u"event"_ns, u"first-input"_ns};
+
 /*
  * Keep this list in alphabetical order.
  * https://w3c.github.io/performance-timeline/#supportedentrytypes-attribute
  */
-static const char16_t* const sValidTypeNames[4] = {
-    u"mark",
-    u"measure",
-    u"navigation",
-    u"resource",
+static constexpr nsLiteralString kValidTypeNames[5] = {
+    u"mark"_ns, u"measure"_ns, u"navigation"_ns, u"paint"_ns, u"resource"_ns,
 };
 
 void PerformanceObserver::ReportUnsupportedTypesErrorToConsole(
@@ -149,9 +146,9 @@ void PerformanceObserver::ReportUnsupportedTypesErrorToConsole(
     nsCOMPtr<nsPIDOMWindowInner> ownerWindow = do_QueryInterface(mOwner);
     Document* document = ownerWindow->GetExtantDoc();
     AutoTArray<nsString, 1> params = {aInvalidTypes};
-    nsContentUtils::ReportToConsole(
-        nsIScriptError::warningFlag, NS_LITERAL_CSTRING("DOM"), document,
-        nsContentUtils::eDOM_PROPERTIES, msgId, params);
+    nsContentUtils::ReportToConsole(nsIScriptError::warningFlag, "DOM"_ns,
+                                    document, nsContentUtils::eDOM_PROPERTIES,
+                                    msgId, params);
   }
 }
 
@@ -168,14 +165,14 @@ void PerformanceObserver::Observe(const PerformanceObserverInit& aOptions,
 
   if (!maybeEntryTypes.WasPassed() && !maybeType.WasPassed()) {
     /* Per spec (3.3.1.2), this should be a syntax error. */
-    aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
+    aRv.ThrowTypeError("Can't call observe without `type` or `entryTypes`");
     return;
   }
 
   if (maybeEntryTypes.WasPassed() &&
       (maybeType.WasPassed() || maybeBuffered.WasPassed())) {
     /* Per spec (3.3.1.3), this, too, should be a syntax error. */
-    aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
+    aRv.ThrowTypeError("Can't call observe with both `type` and `entryTypes`");
     return;
   }
 
@@ -210,11 +207,17 @@ void PerformanceObserver::Observe(const PerformanceObserverInit& aOptions,
 
     /* 3.3.1.5.2 */
     nsTArray<nsString> validEntryTypes;
-    for (const char16_t* name : sValidTypeNames) {
-      nsDependentString validTypeName(name);
-      if (entryTypes.Contains<nsString>(validTypeName) &&
-          !validEntryTypes.Contains<nsString>(validTypeName)) {
-        validEntryTypes.AppendElement(validTypeName);
+
+    if (StaticPrefs::dom_enable_event_timing()) {
+      for (const nsLiteralString& name : kValidEventTimingNames) {
+        if (entryTypes.Contains(name) && !validEntryTypes.Contains(name)) {
+          validEntryTypes.AppendElement(name);
+        }
+      }
+    }
+    for (const nsLiteralString& name : kValidTypeNames) {
+      if (entryTypes.Contains(name) && !validEntryTypes.Contains(name)) {
+        validEntryTypes.AppendElement(name);
       }
     }
 
@@ -257,9 +260,16 @@ void PerformanceObserver::Observe(const PerformanceObserverInit& aOptions,
     nsString type = maybeType.Value();
 
     /* 3.3.1.6.2 */
-    for (const char16_t* name : sValidTypeNames) {
-      nsDependentString validTypeName(name);
-      if (type == validTypeName) {
+    if (StaticPrefs::dom_enable_event_timing()) {
+      for (const nsLiteralString& name : kValidEventTimingNames) {
+        if (type == name) {
+          typeValid = true;
+          break;
+        }
+      }
+    }
+    for (const nsLiteralString& name : kValidTypeNames) {
+      if (type == name) {
         typeValid = true;
         break;
       }
@@ -285,12 +295,12 @@ void PerformanceObserver::Observe(const PerformanceObserverInit& aOptions,
     if (!didUpdateOptionsList) {
       updatedOptionsList.AppendElement(aOptions);
     }
-    mOptions.SwapElements(updatedOptionsList);
+    mOptions = std::move(updatedOptionsList);
 
     /* 3.3.1.6.5 */
     if (maybeBuffered.WasPassed() && maybeBuffered.Value()) {
       nsTArray<RefPtr<PerformanceEntry>> existingEntries;
-      mPerformance->GetEntriesByType(type, existingEntries);
+      mPerformance->GetEntriesByTypeForObserver(type, existingEntries);
       if (!existingEntries.IsEmpty()) {
         mQueuedEntries.AppendElements(existingEntries);
         needQueueNotificationObserverTask = true;
@@ -313,9 +323,13 @@ void PerformanceObserver::GetSupportedEntryTypes(
   nsTArray<nsString> validTypes;
   JS::Rooted<JS::Value> val(aGlobal.Context());
 
-  for (const char16_t* name : sValidTypeNames) {
-    nsString validTypeName(name);
-    validTypes.AppendElement(validTypeName);
+  if (StaticPrefs::dom_enable_event_timing()) {
+    for (const nsLiteralString& name : kValidEventTimingNames) {
+      validTypes.AppendElement(name);
+    }
+  }
+  for (const nsLiteralString& name : kValidTypeNames) {
+    validTypes.AppendElement(name);
   }
 
   if (!ToJSValue(aGlobal.Context(), validTypes, &val)) {
@@ -330,14 +344,8 @@ void PerformanceObserver::GetSupportedEntryTypes(
 
 bool PerformanceObserver::ObservesTypeOfEntry(PerformanceEntry* aEntry) {
   for (auto& option : mOptions) {
-    if (option.mType.WasPassed()) {
-      if (option.mType.Value() == aEntry->GetEntryType()) {
-        return true;
-      }
-    } else {
-      if (option.mEntryTypes.Value().Contains(aEntry->GetEntryType())) {
-        return true;
-      }
+    if (aEntry->ShouldAddEntryToObserverBuffer(option)) {
+      return true;
     }
   }
   return false;
@@ -347,6 +355,7 @@ void PerformanceObserver::Disconnect() {
   if (mConnected) {
     MOZ_ASSERT(mPerformance);
     mPerformance->RemoveObserver(this);
+    mOptions.Clear();
     mConnected = false;
   }
 }
@@ -354,5 +363,5 @@ void PerformanceObserver::Disconnect() {
 void PerformanceObserver::TakeRecords(
     nsTArray<RefPtr<PerformanceEntry>>& aRetval) {
   MOZ_ASSERT(aRetval.IsEmpty());
-  aRetval.SwapElements(mQueuedEntries);
+  aRetval = std::move(mQueuedEntries);
 }
