@@ -6,37 +6,60 @@
 
 #include "QuotaManagerService.h"
 
+// Local includes
 #include "ActorsChild.h"
-#include "mozilla/BasePrincipal.h"
-#include "mozilla/ClearOnShutdown.h"
-#include "mozilla/Hal.h"
-#include "mozilla/Preferences.h"
-#include "mozilla/Unused.h"
-#include "mozilla/ipc/BackgroundChild.h"
-#include "mozilla/ipc/BackgroundParent.h"
-#include "mozilla/ipc/BackgroundUtils.h"
-#include "mozilla/ipc/PBackgroundChild.h"
-#include "nsIIdleService.h"
-#include "nsIObserverService.h"
-#include "nsIScriptSecurityManager.h"
-#include "nsXULAppAPI.h"
+#include "Client.h"
 #include "QuotaManager.h"
 #include "QuotaRequests.h"
 
+// Global includes
+#include <cstdint>
+#include <cstring>
+#include <utility>
+#include "MainThreadUtils.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/Atomics.h"
+#include "mozilla/ClearOnShutdown.h"
+#include "mozilla/Hal.h"
+#include "mozilla/MacroForEach.h"
+#include "mozilla/Maybe.h"
+#include "mozilla/OriginAttributes.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/Services.h"
+#include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/StaticPtr.h"
+#include "mozilla/Unused.h"
+#include "mozilla/Variant.h"
+#include "mozilla/dom/quota/PQuota.h"
+#include "mozilla/dom/quota/PersistenceType.h"
+#include "mozilla/dom/quota/ResultExtensions.h"
+#include "mozilla/fallible.h"
+#include "mozilla/hal_sandbox/PHal.h"
+#include "mozilla/ipc/BackgroundChild.h"
+#include "mozilla/ipc/BackgroundUtils.h"
+#include "mozilla/ipc/PBackgroundChild.h"
+#include "mozilla/ipc/PBackgroundSharedTypes.h"
+#include "nsCOMPtr.h"
+#include "nsContentUtils.h"
+#include "nsDebug.h"
+#include "nsError.h"
+#include "nsIObserverService.h"
+#include "nsIPrincipal.h"
+#include "nsIUserIdleService.h"
+#include "nsServiceManagerUtils.h"
+#include "nsStringFwd.h"
+#include "nsXULAppAPI.h"
+#include "nscore.h"
+
 #define PROFILE_BEFORE_CHANGE_QM_OBSERVER_ID "profile-before-change-qm"
 
-namespace mozilla {
-namespace dom {
-namespace quota {
+namespace mozilla::dom::quota {
 
 using namespace mozilla::ipc;
 
 namespace {
 
-// Preference that is used to enable testing features.
-const char kTestingPref[] = "dom.quotaManager.testing";
-
-const char kIdleServiceContractId[] = "@mozilla.org/widget/idleservice;1";
+const char kIdleServiceContractId[] = "@mozilla.org/widget/useridleservice;1";
 
 // The number of seconds we will wait after receiving the idle-daily
 // notification before beginning maintenance.
@@ -46,15 +69,6 @@ mozilla::StaticRefPtr<QuotaManagerService> gQuotaManagerService;
 
 mozilla::Atomic<bool> gInitialized(false);
 mozilla::Atomic<bool> gClosed(false);
-mozilla::Atomic<bool> gTestingMode(false);
-
-void TestingPrefChangedCallback(const char* aPrefName, void* aClosure) {
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(!strcmp(aPrefName, kTestingPref));
-  MOZ_ASSERT(!aClosure);
-
-  gTestingMode = Preferences::GetBool(aPrefName);
-}
 
 nsresult CheckedPrincipalToPrincipalInfo(nsIPrincipal* aPrincipal,
                                          PrincipalInfo& aPrincipalInfo) {
@@ -79,7 +93,7 @@ nsresult CheckedPrincipalToPrincipalInfo(nsIPrincipal* aPrincipal,
 
 nsresult GetClearResetOriginParams(nsIPrincipal* aPrincipal,
                                    const nsACString& aPersistenceType,
-                                   const nsAString& aClientType, bool aMatchAll,
+                                   const nsAString& aClientType,
                                    ClearResetOriginParams& aParams) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aPrincipal);
@@ -90,34 +104,32 @@ nsresult GetClearResetOriginParams(nsIPrincipal* aPrincipal,
     return rv;
   }
 
-  Nullable<PersistenceType> persistenceType;
-  rv = NullablePersistenceTypeFromText(aPersistenceType, &persistenceType);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  if (persistenceType.IsNull()) {
+  if (aPersistenceType.IsVoid()) {
     aParams.persistenceTypeIsExplicit() = false;
   } else {
-    aParams.persistenceType() = persistenceType.Value();
+    const auto maybePersistenceType =
+        PersistenceTypeFromString(aPersistenceType, fallible);
+    if (NS_WARN_IF(maybePersistenceType.isNothing())) {
+      return NS_ERROR_INVALID_ARG;
+    }
+
+    aParams.persistenceType() = maybePersistenceType.value();
     aParams.persistenceTypeIsExplicit() = true;
   }
 
-  Nullable<Client::Type> clientType;
-  rv = Client::NullableTypeFromText(aClientType, &clientType);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  if (clientType.IsNull()) {
+  if (aClientType.IsVoid()) {
     aParams.clientTypeIsExplicit() = false;
   } else {
-    aParams.clientType() = clientType.Value();
+    Client::Type clientType;
+    bool ok = Client::TypeFromText(aClientType, clientType, fallible);
+    if (NS_WARN_IF(!ok)) {
+      return NS_ERROR_INVALID_ARG;
+    }
+
+    aParams.clientType() = clientType;
     aParams.clientTypeIsExplicit() = true;
   }
 
-  aParams.matchAll() = aMatchAll;
-
   return NS_OK;
 }
 
@@ -130,7 +142,7 @@ class QuotaManagerService::PendingRequestInfo {
  public:
   explicit PendingRequestInfo(RequestBase* aRequest) : mRequest(aRequest) {}
 
-  virtual ~PendingRequestInfo() {}
+  virtual ~PendingRequestInfo() = default;
 
   RequestBase* GetRequest() const { return mRequest; }
 
@@ -265,9 +277,6 @@ nsresult QuotaManagerService::Init() {
     }
   }
 
-  Preferences::RegisterCallbackAndCall(TestingPrefChangedCallback,
-                                       kTestingPref);
-
   return NS_OK;
 }
 
@@ -278,8 +287,6 @@ void QuotaManagerService::Destroy() {
     MOZ_ASSERT(false, "Shutdown more than once?!");
   }
 
-  Preferences::UnregisterCallback(TestingPrefChangedCallback, kTestingPref);
-
   delete this;
 }
 
@@ -316,14 +323,13 @@ nsresult QuotaManagerService::EnsureBackgroundActor() {
   return NS_OK;
 }
 
-nsresult QuotaManagerService::InitiateRequest(
-    nsAutoPtr<PendingRequestInfo>& aInfo) {
+nsresult QuotaManagerService::InitiateRequest(PendingRequestInfo& aInfo) {
   nsresult rv = EnsureBackgroundActor();
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  rv = aInfo->InitiateRequest(mBackgroundActor);
+  rv = aInfo.InitiateRequest(mBackgroundActor);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -369,7 +375,7 @@ void QuotaManagerService::PerformIdleMaintenance() {
     // We don't want user activity to impact this code if we're running tests.
     Unused << Observe(nullptr, OBSERVER_TOPIC_IDLE, nullptr);
   } else if (!mIdleObserverRegistered) {
-    nsCOMPtr<nsIIdleService> idleService =
+    nsCOMPtr<nsIUserIdleService> idleService =
         do_GetService(kIdleServiceContractId);
     MOZ_ASSERT(idleService);
 
@@ -385,7 +391,7 @@ void QuotaManagerService::RemoveIdleObserver() {
   MOZ_ASSERT(NS_IsMainThread());
 
   if (mIdleObserverRegistered) {
-    nsCOMPtr<nsIIdleService> idleService =
+    nsCOMPtr<nsIUserIdleService> idleService =
         do_GetService(kIdleServiceContractId);
     MOZ_ASSERT(idleService);
 
@@ -402,12 +408,84 @@ NS_IMPL_RELEASE_WITH_DESTROY(QuotaManagerService, Destroy())
 NS_IMPL_QUERY_INTERFACE(QuotaManagerService, nsIQuotaManagerService,
                         nsIObserver)
 
+NS_IMETHODIMP
+QuotaManagerService::StorageName(nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(nsContentUtils::IsCallerChrome());
+
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  RefPtr<Request> request = new Request();
+
+  StorageNameParams params;
+
+  RequestInfo info(request, params);
+
+  nsresult rv = InitiateRequest(info);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+QuotaManagerService::StorageInitialized(nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(nsContentUtils::IsCallerChrome());
+
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  RefPtr<Request> request = new Request();
+
+  StorageInitializedParams params;
+
+  RequestInfo info(request, params);
+
+  nsresult rv = InitiateRequest(info);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+QuotaManagerService::TemporaryStorageInitialized(nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(nsContentUtils::IsCallerChrome());
+
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  RefPtr<Request> request = new Request();
+
+  TemporaryStorageInitializedParams params;
+
+  RequestInfo info(request, params);
+
+  nsresult rv = InitiateRequest(info);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 QuotaManagerService::Init(nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(nsContentUtils::IsCallerChrome());
 
-  if (NS_WARN_IF(!gTestingMode)) {
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -415,7 +493,7 @@ QuotaManagerService::Init(nsIQuotaRequest** _retval) {
 
   InitParams params;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -431,7 +509,7 @@ QuotaManagerService::InitTemporaryStorage(nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(nsContentUtils::IsCallerChrome());
 
-  if (NS_WARN_IF(!gTestingMode)) {
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -439,7 +517,7 @@ QuotaManagerService::InitTemporaryStorage(nsIQuotaRequest** _retval) {
 
   InitTemporaryStorageParams params;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -451,19 +529,18 @@ QuotaManagerService::InitTemporaryStorage(nsIQuotaRequest** _retval) {
 }
 
 NS_IMETHODIMP
-QuotaManagerService::InitStoragesForPrincipal(
-    nsIPrincipal* aPrincipal, const nsACString& aPersistenceType,
-    nsIQuotaRequest** _retval) {
+QuotaManagerService::InitializePersistentOrigin(nsIPrincipal* aPrincipal,
+                                                nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(nsContentUtils::IsCallerChrome());
 
-  if (NS_WARN_IF(!gTestingMode)) {
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
     return NS_ERROR_UNEXPECTED;
   }
 
   RefPtr<Request> request = new Request();
 
-  InitOriginParams params;
+  InitializePersistentOriginParams params;
 
   nsresult rv =
       CheckedPrincipalToPrincipalInfo(aPrincipal, params.principalInfo());
@@ -471,15 +548,7 @@ QuotaManagerService::InitStoragesForPrincipal(
     return rv;
   }
 
-  Nullable<PersistenceType> persistenceType;
-  rv = NullablePersistenceTypeFromText(aPersistenceType, &persistenceType);
-  if (NS_WARN_IF(NS_FAILED(rv)) || persistenceType.IsNull()) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  params.persistenceType() = persistenceType.Value();
-
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -490,6 +559,85 @@ QuotaManagerService::InitStoragesForPrincipal(
   return NS_OK;
 }
 
+NS_IMETHODIMP
+QuotaManagerService::InitializeTemporaryOrigin(
+    const nsACString& aPersistenceType, nsIPrincipal* aPrincipal,
+    nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(nsContentUtils::IsCallerChrome());
+
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  RefPtr<Request> request = new Request();
+
+  InitializeTemporaryOriginParams params;
+
+  const auto maybePersistenceType =
+      PersistenceTypeFromString(aPersistenceType, fallible);
+  if (NS_WARN_IF(maybePersistenceType.isNothing())) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  if (NS_WARN_IF(!IsBestEffortPersistenceType(maybePersistenceType.value()))) {
+    return NS_ERROR_FAILURE;
+  }
+
+  params.persistenceType() = maybePersistenceType.value();
+
+  nsresult rv =
+      CheckedPrincipalToPrincipalInfo(aPrincipal, params.principalInfo());
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  RequestInfo info(request, params);
+
+  rv = InitiateRequest(info);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+QuotaManagerService::GetFullOriginMetadata(const nsACString& aPersistenceType,
+                                           nsIPrincipal* aPrincipal,
+                                           nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(nsContentUtils::IsCallerChrome());
+
+  QM_TRY(OkIf(StaticPrefs::dom_quotaManager_testing()), NS_ERROR_UNEXPECTED);
+
+  const auto maybePersistenceType =
+      PersistenceTypeFromString(aPersistenceType, fallible);
+  QM_TRY(OkIf(maybePersistenceType.isSome()), NS_ERROR_INVALID_ARG);
+  QM_TRY(OkIf(IsBestEffortPersistenceType(*maybePersistenceType)),
+         NS_ERROR_INVALID_ARG);
+
+  PrincipalInfo principalInfo;
+  QM_TRY(MOZ_TO_RESULT(PrincipalToPrincipalInfo(aPrincipal, &principalInfo)));
+  QM_TRY(OkIf(QuotaManager::IsPrincipalInfoValid(principalInfo)),
+         NS_ERROR_INVALID_ARG);
+
+  RefPtr<Request> request = new Request();
+
+  GetFullOriginMetadataParams params;
+
+  params.persistenceType() = *maybePersistenceType;
+  params.principalInfo() = std::move(principalInfo);
+
+  RequestInfo info(request, params);
+
+  QM_TRY(MOZ_TO_RESULT(InitiateRequest(info)));
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 QuotaManagerService::GetUsage(nsIQuotaUsageCallback* aCallback, bool aGetAll,
                               nsIQuotaUsageRequest** _retval) {
@@ -502,7 +650,7 @@ QuotaManagerService::GetUsage(nsIQuotaUsageCallback* aCallback, bool aGetAll,
 
   params.getAll() = aGetAll;
 
-  nsAutoPtr<PendingRequestInfo> info(new UsageRequestInfo(request, params));
+  UsageRequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -516,7 +664,7 @@ QuotaManagerService::GetUsage(nsIQuotaUsageCallback* aCallback, bool aGetAll,
 NS_IMETHODIMP
 QuotaManagerService::GetUsageForPrincipal(nsIPrincipal* aPrincipal,
                                           nsIQuotaUsageCallback* aCallback,
-                                          bool aGetGroupUsage,
+                                          bool aFromMemory,
                                           nsIQuotaUsageRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aPrincipal);
@@ -532,9 +680,9 @@ QuotaManagerService::GetUsageForPrincipal(nsIPrincipal* aPrincipal,
     return rv;
   }
 
-  params.getGroupUsage() = aGetGroupUsage;
+  params.fromMemory() = aFromMemory;
 
-  nsAutoPtr<PendingRequestInfo> info(new UsageRequestInfo(request, params));
+  UsageRequestInfo info(request, params);
 
   rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -549,7 +697,7 @@ NS_IMETHODIMP
 QuotaManagerService::Clear(nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
 
-  if (NS_WARN_IF(!gTestingMode)) {
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -557,7 +705,7 @@ QuotaManagerService::Clear(nsIQuotaRequest** _retval) {
 
   ClearAllParams params;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -568,6 +716,23 @@ QuotaManagerService::Clear(nsIQuotaRequest** _retval) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+QuotaManagerService::ClearStoragesForPrivateBrowsing(
+    nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  RefPtr<Request> request = new Request();
+
+  ClearPrivateBrowsingParams params;
+
+  RequestInfo info(request, params);
+
+  QM_TRY(MOZ_TO_RESULT(InitiateRequest(info)));
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 QuotaManagerService::ClearStoragesForOriginAttributesPattern(
     const nsAString& aPattern, nsIQuotaRequest** _retval) {
@@ -582,7 +747,7 @@ QuotaManagerService::ClearStoragesForOriginAttributesPattern(
 
   params.pattern() = pattern;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -614,15 +779,16 @@ QuotaManagerService::ClearStoragesForPrincipal(
   ClearResetOriginParams commonParams;
 
   nsresult rv = GetClearResetOriginParams(aPrincipal, aPersistenceType,
-                                          aClientType, aClearAll, commonParams);
+                                          aClientType, commonParams);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  RequestParams params;
-  params = ClearOriginParams(commonParams);
+  ClearOriginParams params;
+  params.commonParams() = std::move(commonParams);
+  params.matchAll() = aClearAll;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -637,7 +803,7 @@ NS_IMETHODIMP
 QuotaManagerService::Reset(nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
 
-  if (NS_WARN_IF(!gTestingMode)) {
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -645,7 +811,7 @@ QuotaManagerService::Reset(nsIQuotaRequest** _retval) {
 
   ResetAllParams params;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -659,29 +825,20 @@ QuotaManagerService::Reset(nsIQuotaRequest** _retval) {
 NS_IMETHODIMP
 QuotaManagerService::ResetStoragesForPrincipal(
     nsIPrincipal* aPrincipal, const nsACString& aPersistenceType,
-    const nsAString& aClientType, bool aResetAll, nsIQuotaRequest** _retval) {
+    const nsAString& aClientType, nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aPrincipal);
 
-  if (NS_WARN_IF(!gTestingMode)) {
+  if (NS_WARN_IF(!StaticPrefs::dom_quotaManager_testing())) {
     return NS_ERROR_UNEXPECTED;
   }
 
-  nsCString suffix;
-  aPrincipal->OriginAttributesRef().CreateSuffix(suffix);
-
-  if (NS_WARN_IF(aResetAll && !suffix.IsEmpty())) {
-    // The originAttributes should be default originAttributes when the
-    // aClearAll flag is set.
-    return NS_ERROR_INVALID_ARG;
-  }
-
   RefPtr<Request> request = new Request(aPrincipal);
 
   ClearResetOriginParams commonParams;
 
   nsresult rv = GetClearResetOriginParams(aPrincipal, aPersistenceType,
-                                          aClientType, aResetAll, commonParams);
+                                          aClientType, commonParams);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -689,7 +846,7 @@ QuotaManagerService::ResetStoragesForPrincipal(
   RequestParams params;
   params = ResetOriginParams(commonParams);
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -717,7 +874,7 @@ QuotaManagerService::Persisted(nsIPrincipal* aPrincipal,
     return rv;
   }
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -745,7 +902,7 @@ QuotaManagerService::Persist(nsIPrincipal* aPrincipal,
     return rv;
   }
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  RequestInfo info(request, params);
 
   rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -757,16 +914,41 @@ QuotaManagerService::Persist(nsIPrincipal* aPrincipal,
 }
 
 NS_IMETHODIMP
-QuotaManagerService::ListInitializedOrigins(nsIQuotaCallback* aCallback,
-                                            nsIQuotaRequest** _retval) {
+QuotaManagerService::Estimate(nsIPrincipal* aPrincipal,
+                              nsIQuotaRequest** _retval) {
   MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aCallback);
+  MOZ_ASSERT(aPrincipal);
 
-  RefPtr<Request> request = new Request(aCallback);
+  RefPtr<Request> request = new Request(aPrincipal);
 
-  ListInitializedOriginsParams params;
+  EstimateParams params;
 
-  nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, params));
+  nsresult rv =
+      CheckedPrincipalToPrincipalInfo(aPrincipal, params.principalInfo());
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  RequestInfo info(request, params);
+
+  rv = InitiateRequest(info);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  request.forget(_retval);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+QuotaManagerService::ListOrigins(nsIQuotaRequest** _retval) {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  RefPtr<Request> request = new Request();
+
+  ListOriginsParams params;
+
+  RequestInfo info(request, params);
 
   nsresult rv = InitiateRequest(info);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -788,25 +970,13 @@ QuotaManagerService::Observe(nsISupports* aSubject, const char* aTopic,
     return NS_OK;
   }
 
-  if (!strcmp(aTopic, "clear-origin-attributes-data")) {
-    nsCOMPtr<nsIQuotaRequest> request;
-    nsresult rv = ClearStoragesForOriginAttributesPattern(
-        nsDependentString(aData), getter_AddRefs(request));
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return rv;
-    }
-
-    return NS_OK;
-  }
-
   if (!strcmp(aTopic, OBSERVER_TOPIC_IDLE_DAILY)) {
     PerformIdleMaintenance();
     return NS_OK;
   }
 
   if (!strcmp(aTopic, OBSERVER_TOPIC_IDLE)) {
-    nsAutoPtr<PendingRequestInfo> info(
-        new IdleMaintenanceInfo(/* aStart */ true));
+    IdleMaintenanceInfo info(/* aStart */ true);
 
     nsresult rv = InitiateRequest(info);
     if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -819,8 +989,7 @@ QuotaManagerService::Observe(nsISupports* aSubject, const char* aTopic,
   if (!strcmp(aTopic, OBSERVER_TOPIC_ACTIVE)) {
     RemoveIdleObserver();
 
-    nsAutoPtr<PendingRequestInfo> info(
-        new IdleMaintenanceInfo(/* aStart */ false));
+    IdleMaintenanceInfo info(/* aStart */ false);
 
     nsresult rv = InitiateRequest(info);
     if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -891,6 +1060,4 @@ nsresult QuotaManagerService::IdleMaintenanceInfo::InitiateRequest(
   return NS_OK;
 }
 
-}  // namespace quota
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom::quota
