@@ -8,6 +8,7 @@
 
 #include <utility>
 
+#include "ServiceWorkerOpPromise.h"
 #include "js/Exception.h"  // JS::ExceptionStack, JS::StealPendingExceptionStack
 #include "jsapi.h"
 
@@ -19,7 +20,6 @@
 #include "nsIPushErrorReporter.h"
 #include "nsISupportsImpl.h"
 #include "nsITimer.h"
-#include "nsProxyRelease.h"
 #include "nsServiceManagerUtils.h"
 #include "nsTArray.h"
 #include "nsThreadUtils.h"
@@ -46,22 +46,25 @@
 #include "mozilla/dom/Notification.h"
 #include "mozilla/dom/NotificationEvent.h"
 #include "mozilla/dom/NotificationEventBinding.h"
+#include "mozilla/dom/PerformanceTiming.h"
+#include "mozilla/dom/PerformanceStorage.h"
 #include "mozilla/dom/PushEventBinding.h"
 #include "mozilla/dom/RemoteWorkerChild.h"
 #include "mozilla/dom/RemoteWorkerService.h"
 #include "mozilla/dom/Request.h"
 #include "mozilla/dom/Response.h"
 #include "mozilla/dom/RootedDictionary.h"
+#include "mozilla/dom/SafeRefPtr.h"
 #include "mozilla/dom/ServiceWorkerBinding.h"
+#include "mozilla/dom/ServiceWorkerGlobalScopeBinding.h"
 #include "mozilla/dom/WorkerCommon.h"
-#include "mozilla/dom/WorkerPrivate.h"
 #include "mozilla/dom/WorkerRef.h"
 #include "mozilla/dom/WorkerScope.h"
+#include "mozilla/extensions/ExtensionBrowser.h"
 #include "mozilla/ipc/IPCStreamUtils.h"
 #include "mozilla/net/MozURL.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 namespace {
 
@@ -113,11 +116,13 @@ class ExtendableEventKeepAliveHandler final
   /**
    * PromiseNativeHandler interface
    */
-  void ResolvedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue) override {
+  void ResolvedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                        ErrorResult& aRv) override {
     RemovePromise(Resolved);
   }
 
-  void RejectedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue) override {
+  void RejectedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                        ErrorResult& aRv) override {
     RemovePromise(Rejected);
   }
 
@@ -296,12 +301,16 @@ class ServiceWorkerOp::ServiceWorkerOpRunnable : public WorkerDebuggeeRunnable {
   }
 
   nsresult Cancel() override {
+    // We need to check first if cancel is permitted
+    nsresult rv = WorkerRunnable::Cancel();
+    NS_ENSURE_SUCCESS(rv, rv);
+
     MOZ_ASSERT(mOwner);
 
     mOwner->RejectAll(NS_ERROR_DOM_ABORT_ERR);
     mOwner = nullptr;
 
-    return WorkerRunnable::Cancel();
+    return NS_OK;
   }
 
   RefPtr<ServiceWorkerOp> mOwner;
@@ -314,11 +323,11 @@ bool ServiceWorkerOp::MaybeStart(RemoteWorkerChild* aOwner,
                                  RemoteWorkerChild::State& aState) {
   MOZ_ASSERT(!mStarted);
   MOZ_ASSERT(aOwner);
-  MOZ_ASSERT(aOwner->GetOwningEventTarget()->IsOnCurrentThread());
+  MOZ_ASSERT(aOwner->GetActorEventTarget()->IsOnCurrentThread());
 
   auto launcherData = aOwner->mLauncherData.Access();
 
-  if (NS_WARN_IF(!launcherData->mIPCActive)) {
+  if (NS_WARN_IF(!aOwner->CanSend())) {
     RejectAll(NS_ERROR_DOM_ABORT_ERR);
     mStarted = true;
     return true;
@@ -329,8 +338,8 @@ bool ServiceWorkerOp::MaybeStart(RemoteWorkerChild* aOwner,
     return false;
   }
 
-  if (NS_WARN_IF(aState.is<RemoteWorkerChild::PendingTerminated>()) ||
-      NS_WARN_IF(aState.is<RemoteWorkerChild::Terminated>())) {
+  if (NS_WARN_IF(aState.is<RemoteWorkerChild::Canceled>()) ||
+      NS_WARN_IF(aState.is<RemoteWorkerChild::Killed>())) {
     RejectAll(NS_ERROR_DOM_INVALID_STATE_ERR);
     mStarted = true;
     return true;
@@ -358,35 +367,13 @@ bool ServiceWorkerOp::MaybeStart(RemoteWorkerChild* aOwner,
         });
   }
 
+  // NewRunnableMethod doesn't work here because the template does not appear to
+  // be able to deal with the owner argument having storage as a RefPtr but
+  // with the method taking a RefPtr&.
   RefPtr<RemoteWorkerChild> owner = aOwner;
-
   nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
       __func__, [self = std::move(self), owner = std::move(owner)]() mutable {
-        MaybeReportServiceWorkerShutdownProgress(self->mArgs);
-
-        auto lock = owner->mState.Lock();
-        auto& state = lock.ref();
-
-        if (NS_WARN_IF(!state.is<Running>() && !self->IsTerminationOp())) {
-          self->RejectAll(NS_ERROR_DOM_INVALID_STATE_ERR);
-          return;
-        }
-
-        if (self->IsTerminationOp()) {
-          owner->CloseWorkerOnMainThread(state);
-        } else {
-          MOZ_ASSERT(state.is<Running>());
-
-          RefPtr<WorkerRunnable> workerRunnable =
-              self->GetRunnable(state.as<Running>().mWorkerPrivate);
-
-          if (NS_WARN_IF(!workerRunnable->Dispatch())) {
-            self->RejectAll(NS_ERROR_FAILURE);
-          }
-        }
-
-        nsCOMPtr<nsIEventTarget> target = owner->GetOwningEventTarget();
-        NS_ProxyRelease(__func__, target, owner.forget());
+        self->StartOnMainThread(owner);
       });
 
   mStarted = true;
@@ -397,6 +384,33 @@ bool ServiceWorkerOp::MaybeStart(RemoteWorkerChild* aOwner,
   return true;
 }
 
+void ServiceWorkerOp::StartOnMainThread(RefPtr<RemoteWorkerChild>& aOwner) {
+  MaybeReportServiceWorkerShutdownProgress(mArgs);
+
+  {
+    auto lock = aOwner->mState.Lock();
+
+    if (NS_WARN_IF(!lock->is<Running>() && !IsTerminationOp())) {
+      RejectAll(NS_ERROR_DOM_INVALID_STATE_ERR);
+      return;
+    }
+  }
+
+  if (IsTerminationOp()) {
+    aOwner->CloseWorkerOnMainThread();
+  } else {
+    auto lock = aOwner->mState.Lock();
+    MOZ_ASSERT(lock->is<Running>());
+
+    RefPtr<WorkerRunnable> workerRunnable =
+        GetRunnable(lock->as<Running>().mWorkerPrivate);
+
+    if (NS_WARN_IF(!workerRunnable->Dispatch())) {
+      RejectAll(NS_ERROR_FAILURE);
+    }
+  }
+}
+
 void ServiceWorkerOp::Cancel() { RejectAll(NS_ERROR_DOM_ABORT_ERR); }
 
 ServiceWorkerOp::ServiceWorkerOp(
@@ -653,7 +667,7 @@ class PushEventOp final : public ExtendableEventOp {
     const ServiceWorkerPushEventOpArgs& args =
         mArgs.get_ServiceWorkerPushEventOpArgs();
 
-    PushEventInit pushEventInit;
+    RootedDictionary<PushEventInit> pushEventInit(aCx);
 
     if (args.data().type() != OptionalPushData::Tvoid_t) {
       auto& bytes = args.data().get_ArrayOfuint8_t();
@@ -947,7 +961,7 @@ class MessageEventOp final : public ExtendableEventOp {
                  std::function<void(const ServiceWorkerOpResult&)>&& aCallback)
       : ExtendableEventOp(std::move(aArgs), std::move(aCallback)),
         mData(new ServiceWorkerCloneData()) {
-    mData->CopyFromClonedMessageDataForBackgroundChild(
+    mData->CopyFromClonedMessageData(
         mArgs.get_ServiceWorkerMessageEventOpArgs().clonedData());
   }
 
@@ -1065,8 +1079,12 @@ class MOZ_STACK_CLASS FetchEventOp::AutoCancel {
 
       MOZ_ASSERT(!mOwner->mRespondWithPromiseHolder.IsEmpty());
       mOwner->mHandled->MaybeRejectWithNetworkError("AutoCancel"_ns);
-      mOwner->mRespondWithPromiseHolder.Reject(NS_ERROR_INTERCEPTION_FAILED,
-                                               __func__);
+      mOwner->mRespondWithPromiseHolder.Reject(
+          CancelInterceptionArgs(
+              NS_ERROR_INTERCEPTION_FAILED,
+              FetchEventTimeStamps(mOwner->mFetchHandlerStart,
+                                   mOwner->mFetchHandlerFinish)),
+          __func__);
     }
   }
 
@@ -1196,19 +1214,29 @@ void FetchEventOp::ReportCanceled(const nsCString& aPreventDefaultScriptSpec,
 }
 
 FetchEventOp::~FetchEventOp() {
-  mRespondWithPromiseHolder.RejectIfExists(NS_ERROR_DOM_ABORT_ERR, __func__);
-
-  if (mActor) {
-    NS_ProxyRelease("FetchEventOp::mActor", RemoteWorkerService::Thread(),
-                    mActor.forget());
-  }
+  mRespondWithPromiseHolder.RejectIfExists(
+      CancelInterceptionArgs(
+          NS_ERROR_DOM_ABORT_ERR,
+          FetchEventTimeStamps(mFetchHandlerStart, mFetchHandlerFinish)),
+      __func__);
 }
 
 void FetchEventOp::RejectAll(nsresult aStatus) {
   MOZ_ASSERT(!mRespondWithPromiseHolder.IsEmpty());
   MOZ_ASSERT(!mPromiseHolder.IsEmpty());
 
-  mRespondWithPromiseHolder.Reject(aStatus, __func__);
+  if (mFetchHandlerStart.IsNull()) {
+    mFetchHandlerStart = TimeStamp::Now();
+  }
+  if (mFetchHandlerFinish.IsNull()) {
+    mFetchHandlerFinish = TimeStamp::Now();
+  }
+
+  mRespondWithPromiseHolder.Reject(
+      CancelInterceptionArgs(
+          aStatus,
+          FetchEventTimeStamps(mFetchHandlerStart, mFetchHandlerFinish)),
+      __func__);
   mPromiseHolder.Reject(aStatus, __func__);
 }
 
@@ -1240,6 +1268,10 @@ void FetchEventOp::MaybeFinished() {
     // if the worker was terminated before the respondWith promise settled.
 
     mHandled = nullptr;
+    mPreloadResponse = nullptr;
+    mPreloadResponseAvailablePromiseRequestHolder.DisconnectIfExists();
+    mPreloadResponseTimingPromiseRequestHolder.DisconnectIfExists();
+    mPreloadResponseEndPromiseRequestHolder.DisconnectIfExists();
 
     ServiceWorkerFetchEventOpResult result(
         mResult.value() == Resolved ? NS_OK : NS_ERROR_FAILURE);
@@ -1305,19 +1337,25 @@ void FetchEventOp::AsyncLog(const nsCString& aScriptSpec, uint32_t aLineNumber,
 
 void FetchEventOp::GetRequestURL(nsAString& aOutRequestURL) {
   nsTArray<nsCString>& urls =
-      mArgs.get_ServiceWorkerFetchEventOpArgs().internalRequest().urlList();
+      mArgs.get_ParentToChildServiceWorkerFetchEventOpArgs()
+          .common()
+          .internalRequest()
+          .urlList();
   MOZ_ASSERT(!urls.IsEmpty());
 
   CopyUTF8toUTF16(urls.LastElement(), aOutRequestURL);
 }
 
 void FetchEventOp::ResolvedCallback(JSContext* aCx,
-                                    JS::Handle<JS::Value> aValue) {
+                                    JS::Handle<JS::Value> aValue,
+                                    ErrorResult& aRv) {
   MOZ_ASSERT(IsCurrentThreadRunningWorker());
   MOZ_ASSERT(mRespondWithClosure);
   MOZ_ASSERT(!mRespondWithPromiseHolder.IsEmpty());
   MOZ_ASSERT(!mPromiseHolder.IsEmpty());
 
+  mFetchHandlerFinish = TimeStamp::Now();
+
   nsAutoString requestURL;
   GetRequestURL(requestURL);
 
@@ -1377,9 +1415,9 @@ void FetchEventOp::ResolvedCallback(JSContext* aCx,
     return;
   }
 
-  const ServiceWorkerFetchEventOpArgs& args =
-      mArgs.get_ServiceWorkerFetchEventOpArgs();
-  const RequestMode requestMode = args.internalRequest().requestMode();
+  const ParentToChildServiceWorkerFetchEventOpArgs& args =
+      mArgs.get_ParentToChildServiceWorkerFetchEventOpArgs();
+  const RequestMode requestMode = args.common().internalRequest().requestMode();
 
   if (response->Type() == ResponseType::Opaque &&
       requestMode != RequestMode::No_cors) {
@@ -1395,7 +1433,7 @@ void FetchEventOp::ResolvedCallback(JSContext* aCx,
   }
 
   const RequestRedirect requestRedirectMode =
-      args.internalRequest().requestRedirect();
+      args.common().internalRequest().requestRedirect();
 
   if (requestRedirectMode != RequestRedirect::Manual &&
       response->Type() == ResponseType::Opaqueredirect) {
@@ -1411,22 +1449,13 @@ void FetchEventOp::ResolvedCallback(JSContext* aCx,
     return;
   }
 
-  {
-    ErrorResult error;
-    bool bodyUsed = response->GetBodyUsed(error);
-    error.WouldReportJSException();
-    if (NS_WARN_IF(error.Failed())) {
-      autoCancel.SetCancelErrorResult(aCx, error);
-      return;
-    }
-    if (NS_WARN_IF(bodyUsed)) {
-      autoCancel.SetCancelMessage("InterceptedUsedResponseWithURL"_ns,
-                                  requestURL);
-      return;
-    }
+  if (NS_WARN_IF(response->BodyUsed())) {
+    autoCancel.SetCancelMessage("InterceptedUsedResponseWithURL"_ns,
+                                requestURL);
+    return;
   }
 
-  RefPtr<InternalResponse> ir = response->GetInternalResponse();
+  SafeRefPtr<InternalResponse> ir = response->GetInternalResponse();
   if (NS_WARN_IF(!ir)) {
     return;
   }
@@ -1440,8 +1469,6 @@ void FetchEventOp::ResolvedCallback(JSContext* aCx,
     return;
   }
 
-  Telemetry::ScalarAdd(Telemetry::ScalarID::SW_SYNTHESIZED_RES_COUNT, 1);
-
   if (requestMode == RequestMode::Same_origin &&
       response->Type() == ResponseType::Cors) {
     Telemetry::ScalarAdd(Telemetry::ScalarID::SW_CORS_RES_FOR_SO_REQ_COUNT, 1);
@@ -1484,18 +1511,22 @@ void FetchEventOp::ResolvedCallback(JSContext* aCx,
   // where it's going.
   mHandled->MaybeResolveWithUndefined();
   mRespondWithPromiseHolder.Resolve(
-      FetchEventRespondWithResult(
-          SynthesizeResponseArgs(ir, mRespondWithClosure.ref())),
+      FetchEventRespondWithResult(std::make_tuple(
+          std::move(ir), mRespondWithClosure.ref(),
+          FetchEventTimeStamps(mFetchHandlerStart, mFetchHandlerFinish))),
       __func__);
 }
 
 void FetchEventOp::RejectedCallback(JSContext* aCx,
-                                    JS::Handle<JS::Value> aValue) {
+                                    JS::Handle<JS::Value> aValue,
+                                    ErrorResult& aRv) {
   MOZ_ASSERT(IsCurrentThreadRunningWorker());
   MOZ_ASSERT(mRespondWithClosure);
   MOZ_ASSERT(!mRespondWithPromiseHolder.IsEmpty());
   MOZ_ASSERT(!mPromiseHolder.IsEmpty());
 
+  mFetchHandlerFinish = TimeStamp::Now();
+
   FetchEventRespondWithClosure& closure = mRespondWithClosure.ref();
 
   nsCString sourceSpec = closure.respondWithScriptSpec();
@@ -1518,8 +1549,9 @@ void FetchEventOp::RejectedCallback(JSContext* aCx,
   mHandled->MaybeRejectWithNetworkError(
       "FetchEvent.respondWith() Promise rejected"_ns);
   mRespondWithPromiseHolder.Resolve(
-      FetchEventRespondWithResult(
-          CancelInterceptionArgs(NS_ERROR_INTERCEPTION_FAILED)),
+      FetchEventRespondWithResult(CancelInterceptionArgs(
+          NS_ERROR_INTERCEPTION_FAILED,
+          FetchEventTimeStamps(mFetchHandlerStart, mFetchHandlerFinish))),
       __func__);
 }
 
@@ -1530,8 +1562,8 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
   aWorkerPrivate->AssertIsOnWorkerThread();
   MOZ_ASSERT(aWorkerPrivate->IsServiceWorker());
 
-  ServiceWorkerFetchEventOpArgs& args =
-      mArgs.get_ServiceWorkerFetchEventOpArgs();
+  ParentToChildServiceWorkerFetchEventOpArgs& args =
+      mArgs.get_ParentToChildServiceWorkerFetchEventOpArgs();
 
   /**
    * Testing: Failure injection.
@@ -1554,8 +1586,8 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
    *   NS_ERROR_INTERCEPTION_FAILED, and by returning that here we approximate
    *   that failure mode.
    */
-  if (NS_FAILED(args.testingInjectCancellation())) {
-    return args.testingInjectCancellation();
+  if (NS_FAILED(args.common().testingInjectCancellation())) {
+    return args.common().testingInjectCancellation();
   }
 
   /**
@@ -1602,8 +1634,9 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
    * now. Once we implement .targetClientId we can then start exposing
    * .clientId on non-subresource requests as well.  See bug 1487534.
    */
-  if (!args.clientId().IsEmpty() && !internalRequest->IsNavigationRequest()) {
-    fetchEventInit.mClientId = args.clientId();
+  if (!args.common().clientId().IsEmpty() &&
+      !internalRequest->IsNavigationRequest()) {
+    fetchEventInit.mClientId = args.common().clientId();
   }
 
   /*
@@ -1614,9 +1647,10 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
    * to reservedClient’s [resultingClient's] id, and to the empty string
    * otherwise." (Step 18.8)
    */
-  if (!args.resultingClientId().IsEmpty() && args.isNonSubresourceRequest() &&
+  if (!args.common().resultingClientId().IsEmpty() &&
+      args.common().isNonSubresourceRequest() &&
       internalRequest->Destination() != RequestDestination::Report) {
-    fetchEventInit.mResultingClientId = args.resultingClientId();
+    fetchEventInit.mResultingClientId = args.common().resultingClientId();
   }
 
   /**
@@ -1625,8 +1659,81 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
   RefPtr<FetchEvent> fetchEvent =
       FetchEvent::Constructor(globalObject, u"fetch"_ns, fetchEventInit);
   fetchEvent->SetTrusted(true);
-  fetchEvent->PostInit(args.workerScriptSpec(), this);
+  fetchEvent->PostInit(args.common().workerScriptSpec(), this);
   mHandled = fetchEvent->Handled();
+  mPreloadResponse = fetchEvent->PreloadResponse();
+
+  if (args.common().preloadNavigation()) {
+    RefPtr<FetchEventPreloadResponseAvailablePromise> preloadResponsePromise =
+        mActor->GetPreloadResponseAvailablePromise();
+    MOZ_ASSERT(preloadResponsePromise);
+
+    // If preloadResponsePromise has already settled then this callback will get
+    // run synchronously here.
+    RefPtr<FetchEventOp> self = this;
+    preloadResponsePromise
+        ->Then(
+            GetCurrentSerialEventTarget(), __func__,
+            [self, globalObjectAsSupports](
+                SafeRefPtr<InternalResponse>&& aPreloadResponse) {
+              self->mPreloadResponse->MaybeResolve(
+                  MakeRefPtr<Response>(globalObjectAsSupports,
+                                       std::move(aPreloadResponse), nullptr));
+              self->mPreloadResponseAvailablePromiseRequestHolder.Complete();
+            },
+            [self](int) {
+              self->mPreloadResponseAvailablePromiseRequestHolder.Complete();
+            })
+        ->Track(mPreloadResponseAvailablePromiseRequestHolder);
+
+    RefPtr<PerformanceStorage> performanceStorage =
+        aWorkerPrivate->GetPerformanceStorage();
+
+    RefPtr<FetchEventPreloadResponseTimingPromise>
+        preloadResponseTimingPromise =
+            mActor->GetPreloadResponseTimingPromise();
+    MOZ_ASSERT(preloadResponseTimingPromise);
+    preloadResponseTimingPromise
+        ->Then(
+            GetCurrentSerialEventTarget(), __func__,
+            [self, performanceStorage,
+             globalObjectAsSupports](ResponseTiming&& aTiming) {
+              if (performanceStorage && !aTiming.entryName().IsEmpty() &&
+                  aTiming.initiatorType().Equals(u"navigation"_ns)) {
+                performanceStorage->AddEntry(
+                    aTiming.entryName(), aTiming.initiatorType(),
+                    MakeUnique<PerformanceTimingData>(aTiming.timingData()));
+              }
+              self->mPreloadResponseTimingPromiseRequestHolder.Complete();
+            },
+            [self](int) {
+              self->mPreloadResponseTimingPromiseRequestHolder.Complete();
+            })
+        ->Track(mPreloadResponseTimingPromiseRequestHolder);
+
+    RefPtr<FetchEventPreloadResponseEndPromise> preloadResponseEndPromise =
+        mActor->GetPreloadResponseEndPromise();
+    MOZ_ASSERT(preloadResponseEndPromise);
+    preloadResponseEndPromise
+        ->Then(
+            GetCurrentSerialEventTarget(), __func__,
+            [self, globalObjectAsSupports](ResponseEndArgs&& aArgs) {
+              if (aArgs.endReason() == FetchDriverObserver::eAborted) {
+                self->mPreloadResponse->MaybeReject(NS_ERROR_DOM_ABORT_ERR);
+              }
+              self->mPreloadResponseEndPromiseRequestHolder.Complete();
+            },
+            [self](int) {
+              self->mPreloadResponseEndPromiseRequestHolder.Complete();
+            })
+        ->Track(mPreloadResponseEndPromiseRequestHolder);
+  } else {
+    // preload navigation is disabled, resolved preload response promise with
+    // undefined as default behavior.
+    mPreloadResponse->MaybeResolveWithUndefined();
+  }
+
+  mFetchHandlerStart = TimeStamp::Now();
 
   /**
    * Step 5: Dispatch the FetchEvent to the worker's global object
@@ -1637,6 +1744,7 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
 
   if (NS_WARN_IF(dispatchFailed)) {
     mHandled = nullptr;
+    mPreloadResponse = nullptr;
     return rv;
   }
 
@@ -1675,6 +1783,8 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
     MOZ_ASSERT(!aWorkerPrivate->UsesSystemPrincipal(),
                "We don't support system-principal serviceworkers");
 
+    mFetchHandlerFinish = TimeStamp::Now();
+
     if (fetchEvent->DefaultPrevented(CallerType::NonSystem)) {
       // https://w3c.github.io/ServiceWorker/#on-fetch-request-algorithm
       // Step 24.1.1: If eventHandled is not null, then reject eventHandled with
@@ -1682,15 +1792,18 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
       mHandled->MaybeRejectWithNetworkError(
           "FetchEvent.preventDefault() called"_ns);
       mRespondWithPromiseHolder.Resolve(
-          FetchEventRespondWithResult(
-              CancelInterceptionArgs(NS_ERROR_INTERCEPTION_FAILED)),
+          FetchEventRespondWithResult(CancelInterceptionArgs(
+              NS_ERROR_INTERCEPTION_FAILED,
+              FetchEventTimeStamps(mFetchHandlerStart, mFetchHandlerFinish))),
           __func__);
     } else {
       // https://w3c.github.io/ServiceWorker/#on-fetch-request-algorithm
       // Step 24.2: If eventHandled is not null, then resolve eventHandled.
       mHandled->MaybeResolveWithUndefined();
       mRespondWithPromiseHolder.Resolve(
-          FetchEventRespondWithResult(ResetInterceptionArgs()), __func__);
+          FetchEventRespondWithResult(ResetInterceptionArgs(
+              FetchEventTimeStamps(mFetchHandlerStart, mFetchHandlerFinish))),
+          __func__);
     }
   } else {
     MOZ_ASSERT(mRespondWithClosure);
@@ -1702,6 +1815,54 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
   return NS_OK;
 }
 
+class ExtensionAPIEventOp final : public ServiceWorkerOp {
+  using ServiceWorkerOp::ServiceWorkerOp;
+
+ public:
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(ExtensionAPIEventOp, override)
+
+ private:
+  ~ExtensionAPIEventOp() = default;
+
+  bool Exec(JSContext* aCx, WorkerPrivate* aWorkerPrivate) override {
+    MOZ_ASSERT(aWorkerPrivate);
+    aWorkerPrivate->AssertIsOnWorkerThread();
+    MOZ_ASSERT(aWorkerPrivate->IsServiceWorker());
+    MOZ_ASSERT(aWorkerPrivate->ExtensionAPIAllowed());
+    MOZ_ASSERT(!mPromiseHolder.IsEmpty());
+
+    ServiceWorkerExtensionAPIEventOpArgs& args =
+        mArgs.get_ServiceWorkerExtensionAPIEventOpArgs();
+
+    ServiceWorkerExtensionAPIEventOpResult result;
+    result.extensionAPIEventListenerWasAdded() = false;
+
+    if (aWorkerPrivate->WorkerScriptExecutedSuccessfully()) {
+      GlobalObject globalObj(aCx, aWorkerPrivate->GlobalScope()->GetWrapper());
+      RefPtr<ServiceWorkerGlobalScope> scope;
+      UNWRAP_OBJECT(ServiceWorkerGlobalScope, globalObj.Get(), scope);
+      SafeRefPtr<extensions::ExtensionBrowser> extensionAPI =
+          scope->AcquireExtensionBrowser();
+      if (!extensionAPI) {
+        // If the worker script did never access the WebExtension APIs
+        // then we can return earlier, no event listener could have been added.
+        mPromiseHolder.Resolve(result, __func__);
+        return true;
+      }
+      // Check if a listener has been subscribed on the expected WebExtensions
+      // API event.
+      bool hasWakeupListener = extensionAPI->HasWakeupEventListener(
+          args.apiNamespace(), args.apiEventName());
+      result.extensionAPIEventListenerWasAdded() = hasWakeupListener;
+      mPromiseHolder.Resolve(result, __func__);
+    } else {
+      mPromiseHolder.Resolve(result, __func__);
+    }
+
+    return true;
+  }
+};
+
 /* static */ already_AddRefed<ServiceWorkerOp> ServiceWorkerOp::Create(
     ServiceWorkerOpArgs&& aArgs,
     std::function<void(const ServiceWorkerOpResult&)>&& aCallback) {
@@ -1739,9 +1900,13 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
     case ServiceWorkerOpArgs::TServiceWorkerMessageEventOpArgs:
       op = MakeRefPtr<MessageEventOp>(std::move(aArgs), std::move(aCallback));
       break;
-    case ServiceWorkerOpArgs::TServiceWorkerFetchEventOpArgs:
+    case ServiceWorkerOpArgs::TParentToChildServiceWorkerFetchEventOpArgs:
       op = MakeRefPtr<FetchEventOp>(std::move(aArgs), std::move(aCallback));
       break;
+    case ServiceWorkerOpArgs::TServiceWorkerExtensionAPIEventOpArgs:
+      op = MakeRefPtr<ExtensionAPIEventOp>(std::move(aArgs),
+                                           std::move(aCallback));
+      break;
     default:
       MOZ_CRASH("Unknown Service Worker operation!");
       return nullptr;
@@ -1750,5 +1915,4 @@ nsresult FetchEventOp::DispatchFetchEvent(JSContext* aCx,
   return op.forget();
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
