@@ -5,7 +5,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "ServiceWorkerScriptCache.h"
-#include "mozilla/SystemGroup.h"
+
+#include "js/Array.h"               // JS::GetArrayLength
+#include "js/PropertyAndElement.h"  // JS_GetElement
+#include "mozilla/TaskQueue.h"
 #include "mozilla/Unused.h"
 #include "mozilla/dom/CacheBinding.h"
 #include "mozilla/dom/cache/CacheStorage.h"
@@ -16,14 +19,18 @@
 #include "mozilla/dom/WorkerCommon.h"
 #include "mozilla/ipc/BackgroundUtils.h"
 #include "mozilla/ipc/PBackgroundSharedTypes.h"
+#include "mozilla/net/CookieJarSettings.h"
+#include "mozilla/ScopeExit.h"
+#include "mozilla/StaticPrefs_extensions.h"
 #include "nsICacheInfoChannel.h"
-#include "nsIHttpChannelInternal.h"
+#include "nsIHttpChannel.h"
 #include "nsIStreamLoader.h"
 #include "nsIThreadRetargetableRequest.h"
+#include "nsIUUIDGenerator.h"
+#include "nsIXPConnect.h"
 
 #include "nsIInputStreamPump.h"
 #include "nsIPrincipal.h"
-#include "nsIScriptError.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsContentUtils.h"
 #include "nsNetUtil.h"
@@ -34,32 +41,29 @@ using mozilla::dom::cache::Cache;
 using mozilla::dom::cache::CacheStorage;
 using mozilla::ipc::PrincipalInfo;
 
-namespace mozilla {
-namespace dom {
-
-namespace serviceWorkerScriptCache {
+namespace mozilla::dom::serviceWorkerScriptCache {
 
 namespace {
 
-// XXX A sandbox nsIGlobalObject does not preserve its reflector, so |aSandbox|
-// must be kept alive as long as the CacheStorage if you want to ensure that
-// the CacheStorage will continue to work. Failures will manifest as errors
-// like "JavaScript error: , line 0: TypeError: The expression cannot be
-// converted to return the specified type."
-already_AddRefed<CacheStorage> CreateCacheStorage(
-    JSContext* aCx, nsIPrincipal* aPrincipal, ErrorResult& aRv,
-    JS::MutableHandle<JSObject*> aSandbox) {
+already_AddRefed<CacheStorage> CreateCacheStorage(JSContext* aCx,
+                                                  nsIPrincipal* aPrincipal,
+                                                  ErrorResult& aRv) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aPrincipal);
 
   nsIXPConnect* xpc = nsContentUtils::XPConnect();
   MOZ_ASSERT(xpc, "This should never be null!");
-  aRv = xpc->CreateSandbox(aCx, aPrincipal, aSandbox.address());
+  JS::Rooted<JSObject*> sandbox(aCx);
+  aRv = xpc->CreateSandbox(aCx, aPrincipal, sandbox.address());
   if (NS_WARN_IF(aRv.Failed())) {
     return nullptr;
   }
 
-  nsCOMPtr<nsIGlobalObject> sandboxGlobalObject = xpc::NativeGlobal(aSandbox);
+  // This is called when the JSContext is not in a realm, so CreateSandbox
+  // returned an unwrapped global.
+  MOZ_ASSERT(JS_IsGlobalObject(sandbox));
+
+  nsCOMPtr<nsIGlobalObject> sandboxGlobalObject = xpc::NativeGlobal(sandbox);
   if (!sandboxGlobalObject) {
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
@@ -69,14 +73,14 @@ already_AddRefed<CacheStorage> CreateCacheStorage(
   // explicitly fails for private browsing so there should never be
   // a service worker running in private browsing mode.  Therefore if
   // we are purging scripts or running a comparison algorithm we cannot
-  // be in private browing.
+  // be in private browsing.
   //
   // Also, bypass the CacheStorage trusted origin checks.  The ServiceWorker
   // has validated the origin prior to this point.  All the information
   // to revalidate is not available now.
-  return CacheStorage::CreateOnMainThread(
-      cache::CHROME_ONLY_NAMESPACE, sandboxGlobalObject, aPrincipal,
-      false /* private browsing */, true /* force trusted origin */, aRv);
+  return CacheStorage::CreateOnMainThread(cache::CHROME_ONLY_NAMESPACE,
+                                          sandboxGlobalObject, aPrincipal,
+                                          true /* force trusted origin */, aRv);
 }
 
 class CompareManager;
@@ -109,7 +113,7 @@ class CompareNetwork final : public nsIStreamLoaderObserver,
   }
 
   nsresult Initialize(nsIPrincipal* aPrincipal, const nsAString& aURL,
-                      nsILoadGroup* aLoadGroup, Cache* const aCache);
+                      Cache* const aCache);
 
   void Abort();
 
@@ -134,7 +138,9 @@ class CompareNetwork final : public nsIStreamLoaderObserver,
     return internalHeaders.forget();
   }
 
-  UniquePtr<PrincipalInfo> TakePrincipalInfo() { return Move(mPrincipalInfo); }
+  UniquePtr<PrincipalInfo> TakePrincipalInfo() {
+    return std::move(mPrincipalInfo);
+  }
 
   bool Succeeded() const { return NS_SUCCEEDED(mNetworkResult); }
 
@@ -202,11 +208,11 @@ class CompareCache final : public PromiseNativeHandler,
 
   void Abort();
 
-  virtual void ResolvedCallback(JSContext* aCx,
-                                JS::Handle<JS::Value> aValue) override;
+  virtual void ResolvedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                                ErrorResult& aRv) override;
 
-  virtual void RejectedCallback(JSContext* aCx,
-                                JS::Handle<JS::Value> aValue) override;
+  virtual void RejectedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                                ErrorResult& aRv) override;
 
   const nsString& Buffer() const {
     MOZ_ASSERT(NS_IsMainThread());
@@ -248,17 +254,20 @@ class CompareManager final : public PromiseNativeHandler {
         mLoadFlags(nsIChannel::LOAD_BYPASS_SERVICE_WORKER),
         mState(WaitingForInitialization),
         mPendingCount(0),
+        mOnFailure(OnFailure::DoNothing),
         mAreScriptsEqual(true) {
     MOZ_ASSERT(NS_IsMainThread());
     MOZ_ASSERT(aRegistration);
   }
 
   nsresult Initialize(nsIPrincipal* aPrincipal, const nsAString& aURL,
-                      const nsAString& aCacheName, nsILoadGroup* aLoadGroup);
+                      const nsAString& aCacheName);
 
-  void ResolvedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue) override;
+  void ResolvedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                        ErrorResult& aRv) override;
 
-  void RejectedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue) override;
+  void RejectedCallback(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                        ErrorResult& aRv) override;
 
   CacheStorage* CacheStorage_() {
     MOZ_ASSERT(NS_IsMainThread());
@@ -295,8 +304,8 @@ class CompareManager final : public PromiseNativeHandler {
 
     if (mAreScriptsEqual) {
       MOZ_ASSERT(mCallback);
-      mCallback->ComparisonResult(aStatus, true /* aSameScripts */,
-                                  EmptyString(), mMaxScope, mLoadFlags);
+      mCallback->ComparisonResult(aStatus, true /* aSameScripts */, mOnFailure,
+                                  u""_ns, mMaxScope, mLoadFlags);
       Cleanup();
       return;
     }
@@ -327,7 +336,7 @@ class CompareManager final : public PromiseNativeHandler {
     mCNList.AppendElement(cn);
     mPendingCount += 1;
 
-    nsresult rv = cn->Initialize(mPrincipal, aURL, mLoadGroup, aCache);
+    nsresult rv = cn->Initialize(mPrincipal, aURL, aCache);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -386,7 +395,7 @@ class CompareManager final : public PromiseNativeHandler {
     }
 
     uint32_t len = 0;
-    if (!JS_GetArrayLength(aCx, obj, &len)) {
+    if (!JS::GetArrayLength(aCx, obj, &len)) {
       return;
     }
 
@@ -395,16 +404,10 @@ class CompareManager final : public PromiseNativeHandler {
 
     mState = WaitingForScriptOrComparisonResult;
 
-    // Always make sure to fetch the main script.  If the old cache has
-    // no entries or the main script entry is missing, then the loop below
-    // may not trigger it.  This should not really happen, but we handle it
-    // gracefully if it does occur.  Its possible the bad cache state is due
-    // to a crash or shutdown during an update, etc.
-    rv = FetchScript(mURL, true /* aIsMainScript */, mOldCache);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return;
-    }
+    bool hasMainScript = false;
+    AutoTArray<nsString, 8> urlList;
 
+    // Extract the list of URLs in the old cache.
     for (uint32_t i = 0; i < len; ++i) {
       JS::Rooted<JS::Value> val(aCx);
       if (NS_WARN_IF(!JS_GetElement(aCx, obj, i, &val)) ||
@@ -418,15 +421,41 @@ class CompareManager final : public PromiseNativeHandler {
         return;
       };
 
-      nsString URL;
-      request->GetUrl(URL);
+      nsString url;
+      request->GetUrl(url);
 
+      if (!hasMainScript && url == mURL) {
+        hasMainScript = true;
+      }
+
+      urlList.AppendElement(url);
+    }
+
+    // If the main script is missing, then something has gone wrong.  We
+    // will try to continue with the update process to trigger a new
+    // installation.  If that fails, however, then uninstall the registration
+    // because it is broken in a way that cannot be fixed.
+    if (!hasMainScript) {
+      mOnFailure = OnFailure::Uninstall;
+    }
+
+    // Always make sure to fetch the main script.  If the old cache has
+    // no entries or the main script entry is missing, then the loop below
+    // may not trigger it.  This should not really happen, but we handle it
+    // gracefully if it does occur.  Its possible the bad cache state is due
+    // to a crash or shutdown during an update, etc.
+    rv = FetchScript(mURL, true /* aIsMainScript */, mOldCache);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return;
+    }
+
+    for (const auto& url : urlList) {
       // We explicitly start the fetch for the main script above.
-      if (mURL == URL) {
+      if (mURL == url) {
         continue;
       }
 
-      rv = FetchScript(URL, false /* aIsMainScript */, mOldCache);
+      rv = FetchScript(url, false /* aIsMainScript */, mOldCache);
       if (NS_WARN_IF(NS_FAILED(rv))) {
         return;
       }
@@ -510,39 +539,39 @@ class CompareManager final : public PromiseNativeHandler {
       return NS_OK;
     }
 
-    ErrorResult result;
     nsCOMPtr<nsIInputStream> body;
-    result = NS_NewCStringInputStream(getter_AddRefs(body),
-                                      NS_ConvertUTF16toUTF8(aCN->Buffer()));
-    if (NS_WARN_IF(result.Failed())) {
-      MOZ_ASSERT(!result.IsErrorWithMessage());
-      return result.StealNSResult();
+    nsresult rv = NS_NewCStringInputStream(
+        getter_AddRefs(body), NS_ConvertUTF16toUTF8(aCN->Buffer()));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
     }
 
-    RefPtr<InternalResponse> ir =
-        new InternalResponse(200, NS_LITERAL_CSTRING("OK"));
+    SafeRefPtr<InternalResponse> ir =
+        MakeSafeRefPtr<InternalResponse>(200, "OK"_ns);
     ir->SetBody(body, aCN->Buffer().Length());
     ir->SetURLList(aCN->URLList());
 
     ir->InitChannelInfo(aCN->GetChannelInfo());
     UniquePtr<PrincipalInfo> principalInfo = aCN->TakePrincipalInfo();
     if (principalInfo) {
-      ir->SetPrincipalInfo(Move(principalInfo));
+      ir->SetPrincipalInfo(std::move(principalInfo));
     }
 
     RefPtr<InternalHeaders> internalHeaders = aCN->GetInternalHeaders();
     ir->Headers()->Fill(*(internalHeaders.get()), IgnoreErrors());
 
     RefPtr<Response> response =
-        new Response(aCache->GetGlobalObject(), ir, nullptr);
+        new Response(aCache->GetGlobalObject(), std::move(ir), nullptr);
 
     RequestOrUSVString request;
-    request.SetAsUSVString().Rebind(aCN->URL().Data(), aCN->URL().Length());
+    request.SetAsUSVString().ShareOrDependUpon(aCN->URL());
 
     // For now we have to wait until the Put Promise is fulfilled before we can
     // continue since Cache does not yet support starting a read that is being
     // written to.
+    ErrorResult result;
     RefPtr<Promise> cachePromise = aCache->Put(aCx, request, *response, result);
+    result.WouldReportJSException();
     if (NS_WARN_IF(result.Failed())) {
       // No exception here because there are no ReadableStreams involved here.
       MOZ_ASSERT(!result.IsJSException());
@@ -557,14 +586,12 @@ class CompareManager final : public PromiseNativeHandler {
 
   RefPtr<ServiceWorkerRegistrationInfo> mRegistration;
   RefPtr<CompareCallback> mCallback;
-  JS::PersistentRooted<JSObject*> mSandbox;
   RefPtr<CacheStorage> mCacheStorage;
 
   nsTArray<RefPtr<CompareNetwork>> mCNList;
 
   nsString mURL;
   RefPtr<nsIPrincipal> mPrincipal;
-  RefPtr<nsILoadGroup> mLoadGroup;
 
   // Used for the old cache where saves the old source scripts.
   RefPtr<Cache> mOldCache;
@@ -586,6 +613,7 @@ class CompareManager final : public PromiseNativeHandler {
   } mState;
 
   uint32_t mPendingCount;
+  OnFailure mOnFailure;
   bool mAreScriptsEqual;
 };
 
@@ -593,13 +621,12 @@ NS_IMPL_ISUPPORTS0(CompareManager)
 
 nsresult CompareNetwork::Initialize(nsIPrincipal* aPrincipal,
                                     const nsAString& aURL,
-                                    nsILoadGroup* aLoadGroup,
                                     Cache* const aCache) {
   MOZ_ASSERT(aPrincipal);
   MOZ_ASSERT(NS_IsMainThread());
 
   nsCOMPtr<nsIURI> uri;
-  nsresult rv = NS_NewURI(getter_AddRefs(uri), aURL, nullptr, nullptr);
+  nsresult rv = NS_NewURI(getter_AddRefs(uri), aURL);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -628,20 +655,37 @@ nsresult CompareNetwork::Initialize(nsIPrincipal* aPrincipal,
 
   // Different settings are needed for fetching imported scripts, since they
   // might be cross-origin scripts.
-  uint32_t secFlags = mIsMainScript
-                          ? nsILoadInfo::SEC_REQUIRE_SAME_ORIGIN_DATA_IS_BLOCKED
-                          : nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS;
+  uint32_t secFlags =
+      mIsMainScript ? nsILoadInfo::SEC_REQUIRE_SAME_ORIGIN_DATA_IS_BLOCKED
+                    : nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_INHERITS_SEC_CONTEXT;
 
   nsContentPolicyType contentPolicyType =
       mIsMainScript ? nsIContentPolicy::TYPE_INTERNAL_SERVICE_WORKER
                     : nsIContentPolicy::TYPE_INTERNAL_WORKER_IMPORT_SCRIPTS;
 
+  // Create a new cookieJarSettings.
+  nsCOMPtr<nsICookieJarSettings> cookieJarSettings =
+      mozilla::net::CookieJarSettings::Create(aPrincipal);
+
+  // Populate the partitionKey by using the given prinicpal. The ServiceWorkers
+  // are using the foreign partitioned principal, so we can get the partitionKey
+  // from it and the partitionKey will only exist if it's in the third-party
+  // context. In first-party context, we can still use the uri to set the
+  // partitionKey.
+  if (!aPrincipal->OriginAttributesRef().mPartitionKey.IsEmpty()) {
+    net::CookieJarSettings::Cast(cookieJarSettings)
+        ->SetPartitionKey(aPrincipal->OriginAttributesRef().mPartitionKey);
+  } else {
+    net::CookieJarSettings::Cast(cookieJarSettings)->SetPartitionKey(uri);
+  }
+
   // Note that because there is no "serviceworker" RequestContext type, we can
   // use the TYPE_INTERNAL_SCRIPT content policy types when loading a service
   // worker.
   rv = NS_NewChannel(getter_AddRefs(mChannel), uri, aPrincipal, secFlags,
-                     contentPolicyType, nullptr, /* aPerformanceStorage */
-                     loadGroup, nullptr /* aCallbacks */, mLoadFlags);
+                     contentPolicyType, cookieJarSettings,
+                     nullptr /* aPerformanceStorage */, loadGroup,
+                     nullptr /* aCallbacks */, mLoadFlags);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -654,8 +698,7 @@ nsresult CompareNetwork::Initialize(nsIPrincipal* aPrincipal,
       MOZ_ASSERT(NS_SUCCEEDED(rv));
     }
 
-    rv = httpChannel->SetRequestHeader(NS_LITERAL_CSTRING("Service-Worker"),
-                                       NS_LITERAL_CSTRING("script"),
+    rv = httpChannel->SetRequestHeader("Service-Worker"_ns, "script"_ns,
                                        /* merge */ false);
     MOZ_ASSERT(NS_SUCCEEDED(rv));
   }
@@ -666,7 +709,7 @@ nsresult CompareNetwork::Initialize(nsIPrincipal* aPrincipal,
     return rv;
   }
 
-  rv = mChannel->AsyncOpen2(loader);
+  rv = mChannel->AsyncOpen(loader);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -757,7 +800,7 @@ void CompareNetwork::Abort() {
     mState = Finished;
 
     MOZ_ASSERT(mChannel);
-    mChannel->Cancel(NS_BINDING_ABORTED);
+    mChannel->CancelWithReason(NS_BINDING_ABORTED, "CompareNetwork::Abort"_ns);
     mChannel = nullptr;
 
     if (mCC) {
@@ -768,7 +811,7 @@ void CompareNetwork::Abort() {
 }
 
 NS_IMETHODIMP
-CompareNetwork::OnStartRequest(nsIRequest* aRequest, nsISupports* aContext) {
+CompareNetwork::OnStartRequest(nsIRequest* aRequest) {
   MOZ_ASSERT(NS_IsMainThread());
 
   if (mState == Finished) {
@@ -817,13 +860,12 @@ nsresult CompareNetwork::SetPrincipalInfo(nsIChannel* aChannel) {
     return rv;
   }
 
-  mPrincipalInfo = Move(principalInfo);
+  mPrincipalInfo = std::move(principalInfo);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-CompareNetwork::OnStopRequest(nsIRequest* aRequest, nsISupports* aContext,
-                              nsresult aStatusCode) {
+CompareNetwork::OnStopRequest(nsIRequest* aRequest, nsresult aStatusCode) {
   // Nothing to do here!
   return NS_OK;
 }
@@ -853,8 +895,134 @@ CompareNetwork::OnStreamComplete(nsIStreamLoader* aLoader,
     return NS_OK;
   }
 
+  nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
+  MOZ_ASSERT(channel, "How come we don't have any channel?");
+
+  nsCOMPtr<nsIURI> uri;
+  channel->GetOriginalURI(getter_AddRefs(uri));
+  bool isExtension = uri->SchemeIs("moz-extension");
+
+  if (isExtension &&
+      !StaticPrefs::extensions_backgroundServiceWorker_enabled_AtStartup()) {
+    // Return earlier with error is the worker script is a moz-extension url
+    // but the feature isn't enabled by prefs.
+    return NS_ERROR_FAILURE;
+  }
+
+  if (isExtension) {
+    // NOTE: trying to register any moz-extension use that doesn't ends
+    // with .js/.jsm/.mjs seems to be already completing with an error
+    // in aStatus and they never reach this point.
+
+    // TODO: look into avoid duplicated parts that could be shared with the HTTP
+    // channel scenario.
+    nsCOMPtr<nsIURI> channelURL;
+    rv = channel->GetURI(getter_AddRefs(channelURL));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+
+    nsCString channelURLSpec;
+    MOZ_ALWAYS_SUCCEEDS(channelURL->GetSpec(channelURLSpec));
+
+    // Append the final URL (which for an extension worker script is going to
+    // be a file or jar url).
+    MOZ_DIAGNOSTIC_ASSERT(!mURLList.IsEmpty());
+    if (channelURLSpec != mURLList[0]) {
+      mURLList.AppendElement(channelURLSpec);
+    }
+
+    char16_t* buffer = nullptr;
+    size_t len = 0;
+
+    rv = ScriptLoader::ConvertToUTF16(channel, aString, aLen, u"UTF-8"_ns,
+                                      nullptr, buffer, len);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+
+    mBuffer.Adopt(buffer, len);
+
+    rv = NS_OK;
+    return NS_OK;
+  }
+
   nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(request);
-  MOZ_ASSERT(httpChannel, "How come we don't have an HTTP channel?");
+
+  // Main scripts cannot be redirected successfully, however extensions
+  // may successfuly redirect imported scripts to a moz-extension url
+  // (if listed in the web_accessible_resources manifest property).
+  //
+  // When the service worker is initially registered the imported scripts
+  // will be loaded from the child process (see dom/workers/ScriptLoader.cpp)
+  // and in that case this method will only be called for the main script.
+  //
+  // When a registered worker is loaded again (e.g. when the webpage calls
+  // the ServiceWorkerRegistration's update method):
+  //
+  // - both the main and imported scripts are loaded by the
+  //   CompareManager::FetchScript
+  // - the update requests for the imported scripts will also be calling this
+  //   method and we should expect scripts redirected to an extension script
+  //   to have a null httpChannel.
+  //
+  // The request that triggers this method is:
+  //
+  // - the one that is coming from the network (which may be intercepted by
+  //   WebRequest listeners in extensions and redirected to a web_accessible
+  //   moz-extension url)
+  // - it will then be compared with a previous response that we may have
+  //   in the cache
+  //
+  // When the next service worker update occurs, if the request (for an imported
+  // script) is not redirected by an extension the cache entry is invalidated
+  // and a network request is triggered for the import.
+  if (!httpChannel) {
+    // Redirecting a service worker main script should fail before reaching this
+    // method.
+    // If a main script is somehow redirected, the diagnostic assert will crash
+    // in non-release builds.  Release builds will return an explicit error.
+    MOZ_DIAGNOSTIC_ASSERT(!mIsMainScript,
+                          "Unexpected ServiceWorker main script redirected");
+    if (mIsMainScript) {
+      return NS_ERROR_UNEXPECTED;
+    }
+
+    nsCOMPtr<nsIPrincipal> channelPrincipal;
+
+    nsIScriptSecurityManager* ssm = nsContentUtils::GetSecurityManager();
+    if (!ssm) {
+      return NS_ERROR_UNEXPECTED;
+    }
+
+    nsresult rv = ssm->GetChannelResultPrincipal(
+        channel, getter_AddRefs(channelPrincipal));
+
+    // An extension did redirect a non-MainScript request to a moz-extension url
+    // (in that case the originalURL is the resolved jar URI and so we have to
+    // look to the channel principal instead).
+    if (channelPrincipal->SchemeIs("moz-extension")) {
+      char16_t* buffer = nullptr;
+      size_t len = 0;
+
+      rv = ScriptLoader::ConvertToUTF16(channel, aString, aLen, u"UTF-8"_ns,
+                                        nullptr, buffer, len);
+      if (NS_WARN_IF(NS_FAILED(rv))) {
+        return rv;
+      }
+
+      mBuffer.Adopt(buffer, len);
+
+      return NS_OK;
+    }
+
+    // Make non-release and debug builds to crash if this happens and fail
+    // explicitly on release builds.
+    MOZ_DIAGNOSTIC_ASSERT(false,
+                          "ServiceWorker imported script redirected to an url "
+                          "with an unexpected scheme");
+    return NS_ERROR_UNEXPECTED;
+  }
 
   bool requestSucceeded;
   rv = httpChannel->GetRequestSucceeded(&requestSucceeded);
@@ -882,8 +1050,8 @@ CompareNetwork::OnStreamComplete(nsIStreamLoader* aLoader,
 
   // Note: we explicitly don't check for the return value here, because the
   // absence of the header is not an error condition.
-  Unused << httpChannel->GetResponseHeader(
-      NS_LITERAL_CSTRING("Service-Worker-Allowed"), mMaxScope);
+  Unused << httpChannel->GetResponseHeader("Service-Worker-Allowed"_ns,
+                                           mMaxScope);
 
   // [9.2 Update]4.13, If response's cache state is not "local",
   // set registration's last update check time to the current time
@@ -902,11 +1070,10 @@ CompareNetwork::OnStreamComplete(nsIStreamLoader* aLoader,
     return rv;
   }
 
-  if (!mimeType.LowerCaseEqualsLiteral("text/javascript") &&
-      !mimeType.LowerCaseEqualsLiteral("application/x-javascript") &&
-      !mimeType.LowerCaseEqualsLiteral("application/javascript")) {
+  if (mimeType.IsEmpty() ||
+      !nsContentUtils::IsJavascriptMIMEType(NS_ConvertUTF8toUTF16(mimeType))) {
     ServiceWorkerManager::LocalizeAndReportToAllClients(
-        mRegistration->Scope(), "ServiceWorkerRegisterMimeTypeError",
+        mRegistration->Scope(), "ServiceWorkerRegisterMimeTypeError2",
         nsTArray<nsString>{NS_ConvertUTF8toUTF16(mRegistration->Scope()),
                            NS_ConvertUTF8toUTF16(mimeType), mURL});
     rv = NS_ERROR_DOM_SECURITY_ERR;
@@ -933,9 +1100,8 @@ CompareNetwork::OnStreamComplete(nsIStreamLoader* aLoader,
   char16_t* buffer = nullptr;
   size_t len = 0;
 
-  rv = ScriptLoader::ConvertToUTF16(httpChannel, aString, aLen,
-                                    NS_LITERAL_STRING("UTF-8"), nullptr, buffer,
-                                    len);
+  rv = ScriptLoader::ConvertToUTF16(httpChannel, aString, aLen, u"UTF-8"_ns,
+                                    nullptr, buffer, len);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -957,7 +1123,7 @@ nsresult CompareCache::Initialize(Cache* const aCache, const nsAString& aURL) {
   jsapi.Init();
 
   RequestOrUSVString request;
-  request.SetAsUSVString().Rebind(aURL.Data(), aURL.Length());
+  request.SetAsUSVString().ShareOrDependUpon(aURL);
   ErrorResult error;
   CacheQueryOptions params;
   RefPtr<Promise> promise = aCache->Match(jsapi.cx(), request, params, error);
@@ -989,7 +1155,7 @@ void CompareCache::Abort() {
     mState = Finished;
 
     if (mPump) {
-      mPump->Cancel(NS_BINDING_ABORTED);
+      mPump->CancelWithReason(NS_BINDING_ABORTED, "CompareCache::Abort"_ns);
       mPump = nullptr;
     }
   }
@@ -1013,8 +1179,8 @@ CompareCache::OnStreamComplete(nsIStreamLoader* aLoader, nsISupports* aContext,
   char16_t* buffer = nullptr;
   size_t len = 0;
 
-  nsresult rv = ScriptLoader::ConvertToUTF16(
-      nullptr, aString, aLen, NS_LITERAL_STRING("UTF-8"), nullptr, buffer, len);
+  nsresult rv = ScriptLoader::ConvertToUTF16(nullptr, aString, aLen,
+                                             u"UTF-8"_ns, nullptr, buffer, len);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     Finish(rv, false);
     return rv;
@@ -1027,7 +1193,8 @@ CompareCache::OnStreamComplete(nsIStreamLoader* aLoader, nsISupports* aContext,
 }
 
 void CompareCache::ResolvedCallback(JSContext* aCx,
-                                    JS::Handle<JS::Value> aValue) {
+                                    JS::Handle<JS::Value> aValue,
+                                    ErrorResult& aRv) {
   MOZ_ASSERT(NS_IsMainThread());
 
   switch (mState) {
@@ -1042,7 +1209,8 @@ void CompareCache::ResolvedCallback(JSContext* aCx,
 }
 
 void CompareCache::RejectedCallback(JSContext* aCx,
-                                    JS::Handle<JS::Value> aValue) {
+                                    JS::Handle<JS::Value> aValue,
+                                    ErrorResult& aRv) {
   MOZ_ASSERT(NS_IsMainThread());
 
   if (mState != Finished) {
@@ -1087,7 +1255,7 @@ void CompareCache::ManageValueResult(JSContext* aCx,
                              0,     /* default segsize */
                              0,     /* default segcount */
                              false, /* default closeWhenDone */
-                             SystemGroup::EventTargetFor(TaskCategory::Other));
+                             GetMainThreadSerialEventTarget());
   if (NS_WARN_IF(NS_FAILED(rv))) {
     Finish(rv, false);
     return;
@@ -1100,7 +1268,7 @@ void CompareCache::ManageValueResult(JSContext* aCx,
     return;
   }
 
-  rv = mPump->AsyncRead(loader, nullptr);
+  rv = mPump->AsyncRead(loader);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     mPump = nullptr;
     Finish(rv, false);
@@ -1111,7 +1279,9 @@ void CompareCache::ManageValueResult(JSContext* aCx,
   if (rr) {
     nsCOMPtr<nsIEventTarget> sts =
         do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID);
-    rv = rr->RetargetDeliveryTo(sts);
+    RefPtr<TaskQueue> queue =
+        TaskQueue::Create(sts.forget(), "CompareCache STS Delivery Queue");
+    rv = rr->RetargetDeliveryTo(queue);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       mPump = nullptr;
       Finish(rv, false);
@@ -1122,8 +1292,7 @@ void CompareCache::ManageValueResult(JSContext* aCx,
 
 nsresult CompareManager::Initialize(nsIPrincipal* aPrincipal,
                                     const nsAString& aURL,
-                                    const nsAString& aCacheName,
-                                    nsILoadGroup* aLoadGroup) {
+                                    const nsAString& aCacheName) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aPrincipal);
   MOZ_ASSERT(mPendingCount == 0);
@@ -1134,15 +1303,13 @@ nsresult CompareManager::Initialize(nsIPrincipal* aPrincipal,
 
   mURL = aURL;
   mPrincipal = aPrincipal;
-  mLoadGroup = aLoadGroup;
 
   // Always create a CacheStorage since we want to write the network entry to
   // the cache even if there isn't an existing one.
   AutoJSAPI jsapi;
   jsapi.Init();
   ErrorResult result;
-  mSandbox.init(jsapi.cx());
-  mCacheStorage = CreateCacheStorage(jsapi.cx(), aPrincipal, result, &mSandbox);
+  mCacheStorage = CreateCacheStorage(jsapi.cx(), aPrincipal, result);
   if (NS_WARN_IF(result.Failed())) {
     MOZ_ASSERT(!result.IsErrorWithMessage());
     return result.StealNSResult();
@@ -1181,7 +1348,8 @@ nsresult CompareManager::Initialize(nsIPrincipal* aPrincipal,
 // 4. Put the value in the cache.
 // For this reason we have mState to know what callback we are handling.
 void CompareManager::ResolvedCallback(JSContext* aCx,
-                                      JS::Handle<JS::Value> aValue) {
+                                      JS::Handle<JS::Value> aValue,
+                                      ErrorResult& aRv) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(mCallback);
 
@@ -1200,8 +1368,8 @@ void CompareManager::ResolvedCallback(JSContext* aCx,
     case WaitingForPut:
       MOZ_DIAGNOSTIC_ASSERT(mPendingCount > 0);
       if (--mPendingCount == 0) {
-        mCallback->ComparisonResult(NS_OK, false /* aIsEqual */, mNewCacheName,
-                                    mMaxScope, mLoadFlags);
+        mCallback->ComparisonResult(NS_OK, false /* aIsEqual */, mOnFailure,
+                                    mNewCacheName, mMaxScope, mLoadFlags);
         Cleanup();
       }
       return;
@@ -1211,7 +1379,8 @@ void CompareManager::ResolvedCallback(JSContext* aCx,
 }
 
 void CompareManager::RejectedCallback(JSContext* aCx,
-                                      JS::Handle<JS::Value> aValue) {
+                                      JS::Handle<JS::Value> aValue,
+                                      ErrorResult& aRv) {
   MOZ_ASSERT(NS_IsMainThread());
   switch (mState) {
     case Finished:
@@ -1237,8 +1406,8 @@ void CompareManager::RejectedCallback(JSContext* aCx,
 
 void CompareManager::Fail(nsresult aStatus) {
   MOZ_ASSERT(NS_IsMainThread());
-  mCallback->ComparisonResult(aStatus, false /* aIsEqual */, EmptyString(),
-                              EmptyCString(), mLoadFlags);
+  mCallback->ComparisonResult(aStatus, false /* aIsEqual */, mOnFailure, u""_ns,
+                              ""_ns, mLoadFlags);
   Cleanup();
 }
 
@@ -1272,9 +1441,8 @@ nsresult PurgeCache(nsIPrincipal* aPrincipal, const nsAString& aCacheName) {
   AutoJSAPI jsapi;
   jsapi.Init();
   ErrorResult rv;
-  JS::Rooted<JSObject*> sandboxObject(jsapi.cx());
   RefPtr<CacheStorage> cacheStorage =
-      CreateCacheStorage(jsapi.cx(), aPrincipal, rv, &sandboxObject);
+      CreateCacheStorage(jsapi.cx(), aPrincipal, rv);
   if (NS_WARN_IF(rv.Failed())) {
     return rv.StealNSResult();
   }
@@ -1285,6 +1453,10 @@ nsresult PurgeCache(nsIPrincipal* aPrincipal, const nsAString& aCacheName) {
     return rv.StealNSResult();
   }
 
+  // Set [[PromiseIsHandled]] to ensure that if this promise gets rejected,
+  // we don't end up reporting a rejected promise to the console.
+  MOZ_ALWAYS_TRUE(promise->SetAnyPromiseIsHandled());
+
   // We don't actually care about the result of the delete operation.
   return NS_OK;
 }
@@ -1314,8 +1486,7 @@ nsresult GenerateCacheName(nsAString& aName) {
 
 nsresult Compare(ServiceWorkerRegistrationInfo* aRegistration,
                  nsIPrincipal* aPrincipal, const nsAString& aCacheName,
-                 const nsAString& aURL, CompareCallback* aCallback,
-                 nsILoadGroup* aLoadGroup) {
+                 const nsAString& aURL, CompareCallback* aCallback) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aRegistration);
   MOZ_ASSERT(aPrincipal);
@@ -1324,7 +1495,7 @@ nsresult Compare(ServiceWorkerRegistrationInfo* aRegistration,
 
   RefPtr<CompareManager> cm = new CompareManager(aRegistration, aCallback);
 
-  nsresult rv = cm->Initialize(aPrincipal, aURL, aCacheName, aLoadGroup);
+  nsresult rv = cm->Initialize(aPrincipal, aURL, aCacheName);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -1332,7 +1503,4 @@ nsresult Compare(ServiceWorkerRegistrationInfo* aRegistration,
   return NS_OK;
 }
 
-}  // namespace serviceWorkerScriptCache
-
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom::serviceWorkerScriptCache
