@@ -4,8 +4,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef MOZILLA_SVGLENGTH_H__
-#define MOZILLA_SVGLENGTH_H__
+#ifndef DOM_SVG_SVGLENGTH_H_
+#define DOM_SVG_SVGLENGTH_H_
 
 #include "nsDebug.h"
 #include "nsMathUtils.h"
@@ -105,11 +105,15 @@ class SVGLength {
     return mUnit == dom::SVGLength_Binding::SVG_LENGTHTYPE_PERCENTAGE;
   }
 
-  static bool IsValidUnitType(uint16_t unit) {
-    return unit > dom::SVGLength_Binding::SVG_LENGTHTYPE_UNKNOWN &&
-           unit <= dom::SVGLength_Binding::SVG_LENGTHTYPE_PC;
+  static bool IsValidUnitType(uint16_t aUnitType) {
+    return aUnitType > dom::SVGLength_Binding::SVG_LENGTHTYPE_UNKNOWN &&
+           aUnitType <= dom::SVGLength_Binding::SVG_LENGTHTYPE_PC;
   }
 
+  static void GetUnitString(nsAString& aUnit, uint16_t aUnitType);
+
+  static uint16_t GetUnitTypeForString(const nsAString& aUnit);
+
   /**
    * Returns the number of user units per current unit.
    *
@@ -122,7 +126,9 @@ class SVGLength {
 
  private:
 #ifdef DEBUG
-  bool IsValid() const { return IsFinite(mValue) && IsValidUnitType(mUnit); }
+  bool IsValid() const {
+    return std::isfinite(mValue) && IsValidUnitType(mUnit);
+  }
 #endif
 
   /**
@@ -149,4 +155,4 @@ class SVGLength {
 
 }  // namespace mozilla
 
-#endif  // MOZILLA_SVGLENGTH_H__
+#endif  // DOM_SVG_SVGLENGTH_H_
