@@ -7,13 +7,12 @@
 #include "URLWorker.h"
 
 #include "mozilla/dom/Blob.h"
+#include "mozilla/dom/BlobImpl.h"
 #include "mozilla/dom/BlobURLProtocolHandler.h"
-#include "mozilla/dom/WorkerPrivate.h"
 #include "mozilla/dom/WorkerRunnable.h"
 #include "mozilla/dom/WorkerScope.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 // This class creates an URL from a DOM Blob on the main thread.
 class CreateURLRunnable : public WorkerMainThreadRunnable {
@@ -24,8 +23,7 @@ class CreateURLRunnable : public WorkerMainThreadRunnable {
  public:
   CreateURLRunnable(WorkerPrivate* aWorkerPrivate, BlobImpl* aBlobImpl,
                     nsAString& aURL)
-      : WorkerMainThreadRunnable(aWorkerPrivate,
-                                 NS_LITERAL_CSTRING("URL :: CreateURL")),
+      : WorkerMainThreadRunnable(aWorkerPrivate, "URL :: CreateURL"_ns),
         mBlobImpl(aBlobImpl),
         mURL(aURL) {
     MOZ_ASSERT(aBlobImpl);
@@ -39,8 +37,8 @@ class CreateURLRunnable : public WorkerMainThreadRunnable {
     nsCOMPtr<nsIPrincipal> principal = mWorkerPrivate->GetPrincipal();
 
     nsAutoCString url;
-    nsresult rv =
-        BlobURLProtocolHandler::AddDataEntry(mBlobImpl, principal, url);
+    nsresult rv = BlobURLProtocolHandler::AddDataEntry(
+        mBlobImpl, principal, Some(mWorkerPrivate->AgentClusterId()), url);
 
     if (NS_FAILED(rv)) {
       NS_WARNING("Failed to add data entry for the blob!");
@@ -48,7 +46,7 @@ class CreateURLRunnable : public WorkerMainThreadRunnable {
       return false;
     }
 
-    mURL = NS_ConvertUTF8toUTF16(url);
+    CopyUTF8toUTF16(url, mURL);
     return true;
   }
 };
@@ -60,8 +58,7 @@ class RevokeURLRunnable : public WorkerMainThreadRunnable {
 
  public:
   RevokeURLRunnable(WorkerPrivate* aWorkerPrivate, const nsAString& aURL)
-      : WorkerMainThreadRunnable(aWorkerPrivate,
-                                 NS_LITERAL_CSTRING("URL :: RevokeURL")),
+      : WorkerMainThreadRunnable(aWorkerPrivate, "URL :: RevokeURL"_ns),
         mURL(aURL) {}
 
   bool MainThreadRun() override {
@@ -69,18 +66,9 @@ class RevokeURLRunnable : public WorkerMainThreadRunnable {
 
     NS_ConvertUTF16toUTF8 url(mURL);
 
-    nsIPrincipal* urlPrincipal =
-        BlobURLProtocolHandler::GetDataEntryPrincipal(url);
-
-    nsCOMPtr<nsIPrincipal> principal = mWorkerPrivate->GetPrincipal();
-
-    bool subsumes;
-    if (urlPrincipal &&
-        NS_SUCCEEDED(principal->Subsumes(urlPrincipal, &subsumes)) &&
-        subsumes) {
-      BlobURLProtocolHandler::RemoveDataEntry(url);
-    }
-
+    BlobURLProtocolHandler::RemoveDataEntry(
+        url, mWorkerPrivate->GetPrincipal(),
+        Some(mWorkerPrivate->AgentClusterId()));
     return true;
   }
 };
@@ -93,8 +81,7 @@ class IsValidURLRunnable : public WorkerMainThreadRunnable {
 
  public:
   IsValidURLRunnable(WorkerPrivate* aWorkerPrivate, const nsAString& aURL)
-      : WorkerMainThreadRunnable(aWorkerPrivate,
-                                 NS_LITERAL_CSTRING("URL :: IsValidURL")),
+      : WorkerMainThreadRunnable(aWorkerPrivate, "URL :: IsValidURL"_ns),
         mURL(aURL),
         mValid(false) {}
 
@@ -154,8 +141,8 @@ void URLWorker::RevokeObjectURL(const GlobalObject& aGlobal,
 }
 
 /* static */
-bool URLWorker::IsValidURL(const GlobalObject& aGlobal, const nsAString& aUrl,
-                           ErrorResult& aRv) {
+bool URLWorker::IsValidObjectURL(const GlobalObject& aGlobal,
+                                 const nsAString& aUrl, ErrorResult& aRv) {
   JSContext* cx = aGlobal.Context();
   WorkerPrivate* workerPrivate = GetWorkerPrivateFromContext(cx);
 
@@ -170,5 +157,4 @@ bool URLWorker::IsValidURL(const GlobalObject& aGlobal, const nsAString& aUrl,
   return runnable->IsValidURL();
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
